/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "AeroContact.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string AeroContact::toXML()
{
	return NULL;
}
     
void AeroContact::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered AeroContact::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node aeroNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("aero:infData").item(0);
    
    if (aeroNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no aero extension to parse");
    }

    DOM_Node ensInfoNode = ((DOM_Element*)&aeroNode)->getElementsByTagName("aero:ensInfo").item(0);
	//DOM_Node ensInfoNode = aeroNode.getFirstChild();

	if (ensInfoNode.isNull()) {
		throw epp_XMLException(__FILE__, __LINE__, "Unable to get aero:ensInfo node");
	}
     
    DOM_NodeList aeroNodes = ensInfoNode.getChildNodes();
   
    if(aeroNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty AeroContact extension");
    }

    DOMString aeroPrefix = ensInfoNode.getPrefix();

    for(unsigned int i = 0; i < aeroNodes.getLength(); i++)
	{
		DOM_Node aeroSubNode = aeroNodes.item(i);

		if(aeroSubNode.getNodeName().equals(aeroPrefix + ":registrantGroup")) {
			string registrantGroup;
			getNodeData(aeroSubNode, registrantGroup);
			m_registrantGroup.ref(new epp_string(registrantGroup));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":ensO")) {
			string ensO;
			getNodeData(aeroSubNode, ensO);
			m_ensO.ref(new epp_string(ensO));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":requestType")) {
			string requestType;
			getNodeData(aeroSubNode, requestType);
			m_requestType.ref(new epp_string(requestType));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":registrationType")) {
			string registrationType;
			getNodeData(aeroSubNode, registrationType);
			m_registrationType.ref(new epp_string(registrationType));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":credentialsType")) {
			string credentialsType;
			getNodeData(aeroSubNode, credentialsType);
			m_credentialsType.ref(new epp_string(credentialsType));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":credentialsValue")) {
			string credentialsValue;
			getNodeData(aeroSubNode, credentialsValue);
			m_credentialsValue.ref(new epp_string(credentialsValue));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":codeValue")) {
			string codeValue;
			getNodeData(aeroSubNode, codeValue);
			m_codeValue.ref(new epp_string(codeValue));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":uniqueIdentifier")) {
			string uniqueIdentifier;
			getNodeData(aeroSubNode, uniqueIdentifier);
			m_uniqueIdentifier.ref(new epp_string(uniqueIdentifier));
		} else if(aeroSubNode.getNodeName().equals(aeroPrefix + ":lastCheckedDate")) {
			string lastCheckedDate;
			getNodeData(aeroSubNode, lastCheckedDate);
			m_lastCheckedDate.ref(new epp_string(lastCheckedDate));
		}
	}
}
