/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "AeroDomain.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

void addXMLElement(EPP_output & xml_request, const string & tag, const epp_string_ref value)
{
    xml_request.beginTag(tag);
    xml_request.putCDATA(*value);
    xml_request.endTag(tag);
}

dom_ptr getChildByTag(const dom_ptr & node, const string & tag)
{
    if(node->isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to find parseme-element");
    }
                                                                                                                             
    dom_ptr resChildNode = dom_ptr(node->getFirstChildElement());
                                                                                                                             
    if(resChildNode->isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parseme-element");
    }
                                                                                                                             
    while(!resChildNode->isNull()) {
        if(resChildNode->getNodeName().find(":" + tag) != string::npos) {
            return resChildNode;
        }
                                                                                                                             
        resChildNode = dom_ptr(resChildNode->getNextSiblingElement());
    } // while "resChildNode"
                                                                                                                             
    throw epp_XMLException(__FILE__, __LINE__, "unable to find " + tag + "-element");
}

eppobject::epp::epp_string AeroDomain::toXML()
{
#ifdef RTK_DEBUG
    cout << "AeroDomain::toXML(): entered ["<<*m_command<<"] ["<<*m_ens_auth_id<<"]"<<"] ["<<*m_ens_auth_key<<"]"endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing aero command value"); 
    }

    if (m_ens_info == NULL)
    {
        if (m_ens_auth_id == NULL ) {
            throw epp_XMLException(__FILE__, __LINE__, "Missing aero ens auth id value"); 
        }

        if (m_ens_auth_key == NULL ) {
            throw epp_XMLException(__FILE__, __LINE__, "Missing aero ens auth key value"); 
        }
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string aero_tag = "aero:";
    aero_tag.append(*m_command);
    
    xml_request.beginTag(aero_tag);
    xml_request.putAttribute("xmlns:aero", "urn:ietf:params:xml:ns:aero-1.0");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:ietf:params:xml:ns:aero-1.0 aero-1.0.xsd");

    if (*m_command == string("update"))
    {
        xml_request.beginTag("aero:chg");
    }

    if (m_ens_auth_id != NULL ) {
        addXMLElement(xml_request, "aero:ensAuthID", m_ens_auth_id);
    }

    if (m_ens_auth_key != NULL ) {
        addXMLElement(xml_request, "aero:ensAuthKey", m_ens_auth_key);
    }

    if (*m_command == string("update"))
    {
        xml_request.endTag("aero:chg");
    }

    if (m_ens_info != NULL) {
        xml_request.beginTag("aero:ensInfo");
        if (m_ens_info->m_ens_class_ != NULL) {
            for (epp_string_seq::iterator i = m_ens_info->m_ens_class_->begin(); i != m_ens_info->m_ens_class_->end(); i++) {
                addXMLElement(xml_request, "aero:ensClass", epp_string_ref(new epp_string(*i)));
            }
        }

        if (m_ens_info->m_registrantGroup != NULL) {
            addXMLElement(xml_request, "aero:registrantGroup", m_ens_info->m_registrantGroup);
        }
        if (m_ens_info->m_ensO != NULL) {
            addXMLElement(xml_request, "aero:ensO", m_ens_info->m_ensO);
        }
        if (m_ens_info->m_requestType != NULL) {
            addXMLElement(xml_request, "aero:requestType", m_ens_info->m_requestType);
        }
        if (m_ens_info->m_registrationType != NULL) {
            addXMLElement(xml_request, "aero:registrationType", m_ens_info->m_registrationType);
        }
        if (m_ens_info->m_credentialsType != NULL) {
            addXMLElement(xml_request, "aero:credentialsType", m_ens_info->m_credentialsType);
        }
        if (m_ens_info->m_credentialsValue != NULL) {
            addXMLElement(xml_request, "aero:credentialsValue", m_ens_info->m_credentialsValue);
        }
        if (m_ens_info->m_codeValue != NULL) {
            addXMLElement(xml_request, "aero:codeValue", m_ens_info->m_codeValue);
        }
        if (m_ens_info->m_uniqueIdentifier != NULL) {
            addXMLElement(xml_request, "aero:uniqueIdentifier", m_ens_info->m_uniqueIdentifier);
        }
        xml_request.endTag("aero:ensInfo");
    }

    xml_request.endTag(aero_tag);

#ifdef RTK_DEBUG
    cout << "AeroDomain::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void AeroDomain::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered AeroDomain::fromXML() ["<<xml<<"]"<<endl;
#endif

    dom_ptr doc = createDOMDocument("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    dom_ptr parsemeNode = dom_ptr(doc->getTag("parseme"));

    if(parsemeNode->isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }

    dom_ptr infDataNode = getChildByTag(parsemeNode, "infData");

    if (infDataNode->isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no aero extension to parse");
    }
    string aeroPrefix = infDataNode->getPrefix();

    dom_ptr infChildNode = dom_ptr(infDataNode->getFirstChildElement());
                                                                                                                             
    if(infChildNode->isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty AeroDomain extension");
    }

    while(!infChildNode->isNull()) {
        string nodeName = infChildNode->getNodeName();

        if(nodeName == string(aeroPrefix + ":ensAuthID")) {
            m_ens_auth_id.ref(new epp_string(infChildNode->getNodeData()));
        } 
        else if (nodeName == string(aeroPrefix + ":ensInfo")) {
            dom_ptr ensInfoChildNode = dom_ptr(infChildNode->getFirstChildElement());
            if(ensInfoChildNode ->isNull()) {
                throw epp_XMLException(__FILE__, __LINE__, "Empty EnsInfo Node");
            }

            m_ens_info.ref(new EnsInfo());

            while (!ensInfoChildNode->isNull()) {
                string ensNodeName = ensInfoChildNode->getNodeName();

                if (ensNodeName == string(aeroPrefix + ":ensClass")) {
                    if (m_ens_info->m_ens_class_ == NULL) {
                        m_ens_info->m_ens_class_.ref(new epp_string_seq());
                    }
                    m_ens_info->m_ens_class_->push_back(ensInfoChildNode->getNodeData());
                }
                else if (ensNodeName == string(aeroPrefix + ":registrantGroup")) {
                    m_ens_info->m_registrantGroup.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":ensO")) {
                    m_ens_info->m_ensO.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":requestType")) {
                    m_ens_info->m_requestType.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":registrationType")) {
                    m_ens_info->m_registrationType.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":credentialsType")) {
                    m_ens_info->m_credentialsType.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":credentialsValue")) {
                    m_ens_info->m_credentialsValue.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":codeValue")) {
                    m_ens_info->m_codeValue.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":uniqueIdentifier")) {
                    m_ens_info->m_uniqueIdentifier.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }
                else if (ensNodeName == string(aeroPrefix + ":lastCheckedDate")) {
                    m_ens_info->m_lastCheckedDate.ref(new epp_string(ensInfoChildNode->getNodeData()));
                }

                ensInfoChildNode = dom_ptr(ensInfoChildNode->getNextSiblingElement());
            }
        }

        infChildNode = dom_ptr(infChildNode->getNextSiblingElement());
    }
}

