/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "AsiaCEDContact.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string AsiaCEDContact::toXML()
{
#ifdef RTK_DEBUG
    cout << "AsiaCEDContact::toXML(): entered ["<<*m_command<<"] <<endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing asia command value"); 
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string asia_tag = "asia:";

    asia_tag.append(*m_command);
    
    xml_request.beginTag(asia_tag);
    xml_request.putAttribute("xmlns:asia", "urn:afilias:params:xml:ns:asia-1.0");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:afilias:params:xml:ns:asia-1.0 asia-1.0.xsd");

    if (m_ced_contact_data == NULL) {
        throw epp_XMLException(__FILE__, __LINE__, "asia data need");
    }

    if (*m_ced_contact_data->m_chg) {
        xml_request.beginTag("asia:chg");
    }

    xml_request.beginTag("asia:cedData");

    if (m_ced_contact_data->m_ccLocality != NULL) {
         xml_request.beginTag("asia:ccLocality");
         xml_request.putCDATA(*m_ced_contact_data->m_ccLocality);
         xml_request.endTag("asia:ccLocality");
    }

    if (m_ced_contact_data->m_localitySp != NULL) {
         xml_request.beginTag("asia:localitySp");
         xml_request.putCDATA(*m_ced_contact_data->m_localitySp);
         xml_request.endTag("asia:localitySp");
    }

    if (m_ced_contact_data->m_localityCity != NULL) {
         xml_request.beginTag("asia:localityCity");
         xml_request.putCDATA(*m_ced_contact_data->m_localityCity);
         xml_request.endTag("asia:localityCity");
    }

    if (m_ced_contact_data->m_legalEntityType != NULL) {
         xml_request.beginTag("asia:legalEntityType");
         xml_request.putCDATA(*m_ced_contact_data->m_legalEntityType);
         xml_request.endTag("asia:legalEntityType");
    }

    if (m_ced_contact_data->m_identForm != NULL) {
         xml_request.beginTag("asia:identForm");
         xml_request.putCDATA(*m_ced_contact_data->m_identForm);
         xml_request.endTag("asia:identForm");
    }

    if (m_ced_contact_data->m_identNumber != NULL) {
         xml_request.beginTag("asia:identNumber");
         xml_request.putCDATA(*m_ced_contact_data->m_identNumber);
         xml_request.endTag("asia:identNumber");
    }

    if (m_ced_contact_data->m_otherLEType != NULL) {
         xml_request.beginTag("asia:otherLEType");
         xml_request.putCDATA(*m_ced_contact_data->m_otherLEType);
         xml_request.endTag("asia:otherLEType");
    }

    if (m_ced_contact_data->m_otherIdentForm != NULL) {
         xml_request.beginTag("asia:otherIdentForm");
         xml_request.putCDATA(*m_ced_contact_data->m_otherIdentForm);
         xml_request.endTag("asia:otherIdentForm");
    }

    xml_request.endTag("asia:cedData");
 
    if (*m_ced_contact_data->m_chg) {
         xml_request.endTag("asia:chg");
    }

    xml_request.endTag(asia_tag);

#ifdef RTK_DEBUG
    cout << "AsiaCEDContact::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void AsiaCEDContact::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered AsiaCEDContact::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node asiaNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("asia:infData").item(0);
    
    if (asiaNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no asia extension to parse");
    }
     
    DOM_NodeList asiaNodes = asiaNode.getChildNodes();
   
    if(asiaNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty AsiaCEDContact extension");
    }

    DOMString asiaPrefix = asiaNode.getPrefix();

    for(unsigned int i = 0; i < asiaNodes.getLength(); i++)
	{
		DOM_Node tmpSubNode = asiaNodes.item(i);

                if (tmpSubNode.getNodeName().equals(asiaPrefix + ":cedData")) {
                    DOM_NodeList asiaResChildNode = tmpSubNode.getChildNodes();
 
                    m_ced_contact_data.ref(new AsiaCEDContactData()); 
                    for(unsigned int j = 0;j < asiaResChildNode.getLength();j++) {
                       DOM_Node asiaSubNode = asiaResChildNode.item(j);  

		       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":ccLocality")) {
			   string node_data;
			   getNodeData(asiaSubNode, node_data);
			   m_ced_contact_data->m_ccLocality.ref(new epp_string(node_data));
		       }
                       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":localitySp")) {
                           string node_data;
                           getNodeData(asiaSubNode, node_data);
                           m_ced_contact_data->m_localitySp.ref(new epp_string(node_data));
                       }
                       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":localityCity")) {
                           string node_data;
                           getNodeData(asiaSubNode, node_data);
                           m_ced_contact_data->m_localityCity.ref(new epp_string(node_data));
                       }
		       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":legalEntityType")) {
			   string node_data;
			   getNodeData(asiaSubNode, node_data);
			   m_ced_contact_data->m_legalEntityType.ref(new epp_string(node_data));
		       } 
		       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":identForm")) {
			   string node_data;
			   getNodeData(asiaSubNode, node_data);
			   m_ced_contact_data->m_identForm.ref(new epp_string(node_data));
		       } 
		       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":identNumber")) {
			   string node_data;
			   getNodeData(asiaSubNode, node_data);
			   m_ced_contact_data->m_identNumber.ref(new epp_string(node_data));
		       } 
		       if(asiaSubNode.getNodeName().equals(asiaPrefix + ":otherLEType")) {
			   string node_data;
			   getNodeData(asiaSubNode, node_data);
			   m_ced_contact_data->m_otherLEType.ref(new epp_string(node_data));
		       }
                        if(asiaSubNode.getNodeName().equals(asiaPrefix + ":otherIdentForm")) {
                           string node_data;
                           getNodeData(asiaSubNode, node_data);
                           m_ced_contact_data->m_otherIdentForm.ref(new epp_string(node_data));
                       }
                    }  
                }
	}
}
