/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "AsiaDomain.h"
#include "AsiaContact.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string AsiaDomain::toXML()
{
#ifdef RTK_DEBUG
    cout << "AsiaDomain::toXML(): entered ["<<*m_command<<"] ["<<*m_iprType<<"] ["<<*m_domainRoid<<"] "<<endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing asia command value"); 
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string asia_tag = "asia:";
    asia_tag.append(*m_command);
    
    xml_request.beginTag(asia_tag);
    xml_request.putAttribute("xmlns:asia", "urn:afilias:params:xml:ns:asia-1.0");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:afilias:params:xml:ns:asia-1.0 asia-1.0.xsd");

    if (m_command.getPtr()->compare("create") == 0 || m_command.getPtr()->compare("update") == 0) {
       if (m_chg != NULL) {
          xml_request.beginTag("asia:chg");
       }

       if(m_contacts != NULL) {
          for(asia_contact_seq::iterator i = m_contacts->begin();
          i != m_contacts->end(); i++)
          {
             xml_request.beginTag("asia:contact");
             if(i->m_type != NULL) {
                xml_request.putAttribute("type",*i->m_type);
             }
             if(i->m_contact != NULL) {
                xml_request.putCDATA(*i->m_contact);
             }
           xml_request.endTag("asia:contact");
          }
       } // if "m_contacts"

       if (m_maintainerUrl != NULL) {
           xml_request.beginTag("asia:maintainerUrl");
           xml_request.putCDATA(*m_maintainerUrl);
           xml_request.endTag("asia:maintainerUrl");
       }

       if (m_chg != NULL) {
          xml_request.endTag("asia:chg");
       } 
    } 

    if ((m_command.getPtr()->compare("info") == 0 || m_command.getPtr()->compare("update") == 0) && m_domainRoid != NULL) {
       xml_request.beginTag("asia:domainRoid");
       xml_request.putCDATA(*m_domainRoid);
       xml_request.endTag("asia:domainRoid");
    }

    xml_request.endTag(asia_tag);

#ifdef RTK_DEBUG
    cout << "AsiaDomain::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void AsiaDomain::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered AsiaDomain::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node asiaNodeInf = ((DOM_Element*)&parsemeNode)->getElementsByTagName("asia:infData").item(0);
   
    DOM_Node asiaNodeCre = ((DOM_Element*)&parsemeNode)->getElementsByTagName("asia:creData").item(0);
 
    if (asiaNodeCre.isNull() && asiaNodeInf.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no asia extension to parse");
    }
     
    if (!asiaNodeCre.isNull()) {
       DOM_NodeList asiaNodes = asiaNodeCre.getChildNodes();
   
       if(asiaNodes.getLength() == 0)
       {
           throw epp_XMLException(__FILE__, __LINE__, "Empty AsiaDomain creData extension");
       }

       DOMString asiaPrefix = asiaNodeCre.getPrefix();

       for(unsigned int i = 0; i < asiaNodes.getLength(); i++)
	{
		DOM_Node asiaSubNode = asiaNodes.item(i);

		if(asiaSubNode.getNodeName().equals(asiaPrefix + ":domainRoid")) {
			string domainR;
			getNodeData(asiaSubNode, domainR);
			m_domainRoid.ref(new epp_string(domainR));
		} 
	}
    }

    if (!asiaNodeInf.isNull()) {
        DOM_NodeList asiaNodes = asiaNodeInf.getChildNodes();

       if(asiaNodes.getLength() == 0)
       {
           throw epp_XMLException(__FILE__, __LINE__, "Empty AsiaDomain infData extension");
       }

       DOMString asiaPrefix = asiaNodeInf.getPrefix();
    
       m_contacts.ref(new asia_contact_seq());
       for(unsigned int i = 0; i < asiaNodes.getLength(); i++)
        {
                DOM_Node asiaSubNode = asiaNodes.item(i);

                if(asiaSubNode.getNodeName().equals(asiaPrefix + ":contact")) {
                        string tmp;
                        getNodeData(asiaSubNode, tmp);
                        string type;
                        getNodeData(asiaSubNode.getAttributes().getNamedItem("type"),type);
 
                        AsiaContact contact_;

                        contact_.m_contact.ref(new epp_string(tmp));
 
                        contact_.m_type.ref(new epp_string(type));
                       
                        m_contacts->push_back(contact_);
                }
                 
                if(asiaSubNode.getNodeName().equals(asiaPrefix + ":maintainerUrl")) {
                        string tmp;
                        getNodeData(asiaSubNode, tmp);
                        m_maintainerUrl.ref(new epp_string(tmp));
                }
        }
    }	
}

