/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "IDN.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string IDN::toXML()
{
#ifdef RTK_DEBUG
    cout << "IDN::toXML(): entered ["<<*m_command<<"] ["<<*m_script<<"]"<<endl;
#endif

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string idn_tag = "idn:";
    idn_tag.append(*m_command);
    
    xml_request.beginTag(idn_tag);
    xml_request.putAttribute("xmlns:idn", "urn:ietf:params:xml:ns:idn-1.0");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:ietf:params:xml:ns:idn-1.0 idn-1.0.xsd");

    if (!(isOTNE) && m_script == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing idn script value"); 
    }

	if (m_command.getPtr()->compare("update") == 0)
	{
		xml_request.beginTag("idn:chg");

		xml_request.beginTag("idn:script");
		if (m_script != NULL)
			xml_request.putCDATA(*m_script);
		xml_request.endTag("idn:script");

		xml_request.endTag("idn:chg");
	}
	else
	{
		xml_request.beginTag("idn:script");
		if (m_script != NULL)
			xml_request.putCDATA(*m_script);
		xml_request.endTag("idn:script");
	}

    xml_request.endTag(idn_tag);

#ifdef RTK_DEBUG
    cout << "IDN::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void IDN::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered IDN::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node idnNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("idn:infData").item(0);
    
    if (idnNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no idn extension to parse");
    }
     
    char *idnNodeNameChars = idnNode.getNodeName().transcode();
    string idnNodeName = idnNodeNameChars;
    delete idnNodeNameChars;
    
    unsigned int colonIndex = idnNodeName.find(":");
    idnNodeName.erase(0, colonIndex+1);
    m_command.ref(new epp_string(idnNodeName));
    
    DOM_NodeList idnNodes = idnNode.getChildNodes();
   
    if(idnNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty IDN extension");
    }

    DOMString idnPrefix = idnNode.getPrefix();

    for(unsigned int i = 0; i < idnNodes.getLength(); i++)
    {
        DOM_Node idnSubNode = idnNodes.item(i);

        if(idnSubNode.getNodeName().equals(idnPrefix + ":script")) {
	    string script;
	    getNodeData(idnSubNode, script);
	    m_script.ref(new epp_string(script));
        } else if(idnSubNode.getNodeName().equals(idnPrefix + ":reason")) {
	    string reason;
	    getNodeData(idnSubNode, reason);
	    m_reason.ref(new epp_string(reason));
        }

    }

}


