/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "Ipr.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string Ipr::toXML()
{
#ifdef RTK_DEBUG
    cout << "Ipr::toXML(): entered ["<<*m_command<<"]"<<endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing ipr command value"); 
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string ipr_tag = "ipr:";

    ipr_tag.append(*m_command);
    
    xml_request.beginTag(ipr_tag);
    if ((m_tld->compare("asia")) == 0) {
       xml_request.putAttribute("xmlns:ipr", "urn:afilias:params:xml:ns:ipr-1.0 ");
       xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
       xml_request.putAttribute("xsi:schemaLocation", "urn:afilias:params:xml:ns:ipr-1.0 ipr-1.0.xsd");
    } else {
       xml_request.putAttribute("xmlns:ipr", "urn:afilias:params:xml:ns:ipr-1.1 ");
       xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
       xml_request.putAttribute("xsi:schemaLocation", "urn:afilias:params:xml:ns:ipr-1.1 ipr-1.1.xsd"); 
    }

    if (m_ipr_data == NULL) {
        throw epp_XMLException(__FILE__, __LINE__, "ipr data need");
    }

    if (m_ipr_data->m_name != NULL) {
        xml_request.beginTag("ipr:name");
        xml_request.putCDATA(*m_ipr_data->m_name);
        xml_request.endTag("ipr:name");
    }

    if (m_ipr_data->m_number != NULL) {
        xml_request.beginTag("ipr:number");
        xml_request.putCDATA(*m_ipr_data->m_number);
        xml_request.endTag("ipr:number");
    }

    if (m_ipr_data->m_country != NULL) {
        xml_request.beginTag("ipr:ccLocality");
        xml_request.putCDATA(*m_ipr_data->m_country);
        xml_request.endTag("ipr:ccLocality");
    }

    if (m_ipr_data->m_reg_date != NULL) {
        xml_request.beginTag("ipr:regDate");
        xml_request.putCDATA(*m_ipr_data->m_reg_date);
        xml_request.endTag("ipr:regDate");
    }

    if (m_ipr_data->m_app_date != NULL) {
        xml_request.beginTag("ipr:appDate");
        xml_request.putCDATA(*m_ipr_data->m_app_date);
        xml_request.endTag("ipr:appDate");
    }
 
    if (m_ipr_data->m_class != NULL) {
        xml_request.beginTag("ipr:class");
        xml_request.putCDATA(*m_ipr_data->m_class);
        xml_request.endTag("ipr:class");
    }
    
    if (m_ipr_data->m_entitlement != NULL) {
        xml_request.beginTag("ipr:entitlement");
        xml_request.putCDATA(*m_ipr_data->m_entitlement);
        xml_request.endTag("ipr:entitlement");
    }

    if (m_ipr_data->m_form != NULL) {
        xml_request.beginTag("ipr:form");
        xml_request.putCDATA(*m_ipr_data->m_form);
        xml_request.endTag("ipr:form");
    }
 
    if (m_ipr_data->m_preVerified != NULL) {
        xml_request.beginTag("ipr:preVerified");
        xml_request.putCDATA(*m_ipr_data->m_preVerified);
        xml_request.endTag("ipr:preVerified");
    }

    if (m_ipr_data->m_type != NULL) {
        xml_request.beginTag("ipr:type");
        xml_request.putCDATA(*m_ipr_data->m_type);
        xml_request.endTag("ipr:type");
    }

    xml_request.endTag(ipr_tag);

#ifdef RTK_DEBUG
    cout << "Ipr::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void Ipr::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered Ipr::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node iprNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("ipr:infData").item(0);
    
    if (iprNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no ipr extension to parse");
    }
     
    DOM_NodeList iprNodes = iprNode.getChildNodes();
   
    if(iprNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty Ipr extension");
    }

    m_ipr_data.ref(new IprData());

    DOMString iprPrefix = iprNode.getPrefix();

    for(unsigned int i = 0; i < iprNodes.getLength(); i++)
	{
		DOM_Node iprSubNode = iprNodes.item(i);

		if(iprSubNode.getNodeName().equals(iprPrefix + ":name")) {
			string node_data;
			getNodeData(iprSubNode, node_data);
			m_ipr_data->m_name.ref(new epp_string(node_data));
		} 
		if(iprSubNode.getNodeName().equals(iprPrefix + ":number")) {
			string node_data;
			getNodeData(iprSubNode, node_data);
			m_ipr_data->m_number.ref(new epp_string(node_data));
		} 
		if(iprSubNode.getNodeName().equals(iprPrefix + ":ccLocality")) {
			string node_data;
			getNodeData(iprSubNode, node_data);
			m_ipr_data->m_country.ref(new epp_string(node_data));
		} 
		if(iprSubNode.getNodeName().equals(iprPrefix + ":appDate")) {
			string node_data;
			getNodeData(iprSubNode, node_data);
			m_ipr_data->m_app_date.ref(new epp_string(node_data));
		} 
		if(iprSubNode.getNodeName().equals(iprPrefix + ":regDate")) {
			string node_data;
			getNodeData(iprSubNode, node_data);
			m_ipr_data->m_reg_date.ref(new epp_string(node_data));
		}
                if(iprSubNode.getNodeName().equals(iprPrefix + ":class")) {
                        string node_data;
                        getNodeData(iprSubNode, node_data);
                        m_ipr_data->m_class.ref(new epp_string(node_data));
                }
                if(iprSubNode.getNodeName().equals(iprPrefix + ":entitlement")) {
                        string node_data;
                        getNodeData(iprSubNode, node_data);
                        m_ipr_data->m_entitlement.ref(new epp_string(node_data));
                }
                if(iprSubNode.getNodeName().equals(iprPrefix + ":form")) {
                        string node_data;
                        getNodeData(iprSubNode, node_data);
                        m_ipr_data->m_form.ref(new epp_string(node_data));
                }
                if(iprSubNode.getNodeName().equals(iprPrefix + ":type")) {
                        string node_data;
                        getNodeData(iprSubNode, node_data);
                        m_ipr_data->m_type.ref(new epp_string(node_data));
                }
                if(iprSubNode.getNodeName().equals(iprPrefix + ":preVerified")) {
                        string node_data;
                        getNodeData(iprSubNode, node_data);
                        m_ipr_data->m_preVerified.ref(new epp_string(node_data));
                }     
	}
}
