/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "MobiDomain.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string MobiDomain::toXML()
{
#ifdef RTK_DEBUG
    cout << "MobiDomain::toXML(): entered ["<<*m_command<<"] ["<<*m_maintainer_url<<"]"<<endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing mobi command value"); 
    }

    if (m_maintainer_url == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing mobi maintainer url value"); 
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string mobi_tag = "mobi:";
    mobi_tag.append(*m_command);
    
    xml_request.beginTag(mobi_tag);
    xml_request.putAttribute("xmlns:mobi", "urn:ietf:params:xml:ns:mobi-1.0");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:ietf:params:xml:ns:mobi-1.0 mobi-1.0.xsd");

	xml_request.beginTag("mobi:maintainerUrl");
	xml_request.putCDATA(*m_maintainer_url);
	xml_request.endTag("mobi:maintainerUrl");

    xml_request.endTag(mobi_tag);

#ifdef RTK_DEBUG
    cout << "MobiDomain::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void MobiDomain::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered MobiDomain::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node mobiNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("mobi:infData").item(0);
    
    if (mobiNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no mobi extension to parse");
    }
     
    DOM_NodeList mobiNodes = mobiNode.getChildNodes();
   
    if(mobiNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty MobiDomain extension");
    }

    DOMString mobiPrefix = mobiNode.getPrefix();

    for(unsigned int i = 0; i < mobiNodes.getLength(); i++)
	{
		DOM_Node mobiSubNode = mobiNodes.item(i);

		if(mobiSubNode.getNodeName().equals(mobiPrefix + ":maintainerUrl")) {
			string ens_auth_id;
			getNodeData(mobiSubNode, ens_auth_id);
			m_maintainer_url.ref(new epp_string(ens_auth_id));
		} 
	}
}
