/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "OxrsDomain.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string OxrsDomain::toXML()
{
#ifdef RTK_DEBUG
    cout << "OxrsDomain::toXML(): entered ["<<*m_command<<"] ["<<*m_iprType<<"] ["<<*m_domainRoid<<"] "<<endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing oxrs command value"); 
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string oxrs_tag = "oxrs:";
    oxrs_tag.append(*m_command);
    
    xml_request.beginTag(oxrs_tag);
    xml_request.putAttribute("xmlns:oxrs", "urn:afilias:params:xml:ns:oxrs-1.1");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:afilias:params:xml:ns:oxrs-1.1 oxrs-1.1.xsd");

    if (m_command.getPtr()->compare("create") == 0 || m_command.getPtr()->compare("update") == 0) {
       if (m_chg != NULL) {
          xml_request.beginTag("oxrs:chg");
       }

       if (m_maintainerUrl != NULL) {
           xml_request.beginTag("oxrs:maintainerUrl");
           xml_request.putCDATA(*m_maintainerUrl);
           xml_request.endTag("oxrs:maintainerUrl");
       }

       if (m_chg != NULL) {
          xml_request.endTag("oxrs:chg");
       } 
    } 

    if ((m_command.getPtr()->compare("info") == 0 || m_command.getPtr()->compare("update") == 0) && m_domainRoid != NULL) {
       xml_request.beginTag("oxrs:domainRoid");
       xml_request.putCDATA(*m_domainRoid);
       xml_request.endTag("oxrs:domainRoid");
    }

    xml_request.endTag(oxrs_tag);

#ifdef RTK_DEBUG
    cout << "OxrsDomain::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void OxrsDomain::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered OxrsDomain::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node oxrsNodeInf = ((DOM_Element*)&parsemeNode)->getElementsByTagName("oxrs:infData").item(0);
   
    DOM_Node oxrsNodeCre = ((DOM_Element*)&parsemeNode)->getElementsByTagName("oxrs:creData").item(0);
 
    if (oxrsNodeCre.isNull() && oxrsNodeInf.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no oxrs extension to parse");
    }
     
    if (!oxrsNodeCre.isNull()) {
       DOM_NodeList oxrsNodes = oxrsNodeCre.getChildNodes();
   
       if(oxrsNodes.getLength() == 0)
       {
           throw epp_XMLException(__FILE__, __LINE__, "Empty OxrsDomain creData extension");
       }

       DOMString oxrsPrefix = oxrsNodeCre.getPrefix();

       for(unsigned int i = 0; i < oxrsNodes.getLength(); i++)
	{
		DOM_Node oxrsSubNode = oxrsNodes.item(i);

		if(oxrsSubNode.getNodeName().equals(oxrsPrefix + ":domainRoid")) {
			string domainR;
			getNodeData(oxrsSubNode, domainR);
			m_domainRoid.ref(new epp_string(domainR));
		} 
	}
    }

    if (!oxrsNodeInf.isNull()) {
        DOM_NodeList oxrsNodes = oxrsNodeInf.getChildNodes();

       if(oxrsNodes.getLength() == 0)
       {
           throw epp_XMLException(__FILE__, __LINE__, "Empty OxrsDomain infData extension");
       }

       DOMString oxrsPrefix = oxrsNodeInf.getPrefix();
    
       for(unsigned int i = 0; i < oxrsNodes.getLength(); i++)
        {
                DOM_Node oxrsSubNode = oxrsNodes.item(i);

                if(oxrsSubNode.getNodeName().equals(oxrsPrefix + ":maintainerUrl")) {
                        string tmp;
                        getNodeData(oxrsSubNode, tmp);
                        m_maintainerUrl.ref(new epp_string(tmp));
                }
        }
    }	
}

