/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "Rgp.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string Rgp::toXML()
{
#ifdef RTK_DEBUG
    cout << "Rgp::toXML(): entered"<<endl;
#endif

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    xml_request.beginTag("rgp:update");
    xml_request.putAttribute("xmlns:rgp", "urn:ietf:params:xml:ns:rgp-1.0");
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:ietf:params:xml:ns:rgp-1.0 rgp-1.0.xsd");

    if (*m_restore_op == string("report") && m_rgp_report_data == NULL) {
        throw epp_XMLException(__FILE__, __LINE__, "rgp report data needed with op report");
    }

    if (m_restore_op != NULL && *m_restore_op == string("request")) {
        xml_request.beginTag("rgp:restore");
        xml_request.putAttribute("op", *m_restore_op);
        xml_request.endTag("rgp:restore");    
    } else if (*m_restore_op == string("report")) {
        xml_request.beginTag("rgp:restore");
        xml_request.putAttribute("op", *m_restore_op);
    }
   
    if (m_rgp_report_data != NULL) {
       xml_request.beginTag("rgp:report");
 
       if (m_rgp_report_data->m_pre_data != NULL) {
           xml_request.beginTag("rgp:preData");
           xml_request.putCDATA(*m_rgp_report_data->m_pre_data);
           xml_request.endTag("rgp:preData");
       }
       if (m_rgp_report_data->m_post_data != NULL) {
           xml_request.beginTag("rgp:postData");
           xml_request.putCDATA(*m_rgp_report_data->m_post_data);
           xml_request.endTag("rgp:postData");
       }       
       if (m_rgp_report_data->m_del_time != NULL) {
           xml_request.beginTag("rgp:delTime");
           xml_request.putCDATA(*m_rgp_report_data->m_del_time);
           xml_request.endTag("rgp:delTime");
       }
       if (m_rgp_report_data->m_res_time != NULL) {
           xml_request.beginTag("rgp:resTime");
           xml_request.putCDATA(*m_rgp_report_data->m_res_time);
           xml_request.endTag("rgp:resTime");
       }
       if (m_rgp_report_data->m_res_reason != NULL) {
           xml_request.beginTag("rgp:resReason");
           xml_request.putCDATA(*m_rgp_report_data->m_res_reason);
           xml_request.endTag("rgp:resReason");
       }
       if (m_rgp_report_data->m_statement_1 != NULL) {
           xml_request.beginTag("rgp:statement");
           xml_request.putCDATA(*m_rgp_report_data->m_statement_1);
           xml_request.endTag("rgp:statement");
       }
       if (m_rgp_report_data->m_statement_2 != NULL) {
           xml_request.beginTag("rgp:statement");
           xml_request.putCDATA(*m_rgp_report_data->m_statement_2);
           xml_request.endTag("rgp:statement");
       }
       if (m_rgp_report_data->m_other != NULL) {
           xml_request.beginTag("rgp:other");
           xml_request.putCDATA(*m_rgp_report_data->m_other);
           xml_request.endTag("rgp:other");
       }

       xml_request.endTag("rgp:report");
    } 

    if (m_restore_op != NULL && *m_restore_op == string("report")) {
       xml_request.endTag("rgp:restore");
    }

    xml_request.endTag("rgp:update");

#ifdef RTK_DEBUG
    cout << "Rgp::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void Rgp::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered Rgp::fromXML() ["<<xml<<"]"<<endl;
#endif
    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }

    DOM_Node rgpNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("rgp:infData").item(0);

    if (rgpNode.isNull())
        rgpNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("rgp:upData").item(0);
 
    if (rgpNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no rgp extension to parse");
    }

    DOM_NodeList rgpNodes = rgpNode.getChildNodes();

    if(rgpNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty Rgp extension");
    }

    DOMString rgpPrefix = rgpNode.getPrefix();

    for(unsigned int i = 0; i < rgpNodes.getLength(); i++)
    {
           DOM_Node rgpSubNode = rgpNodes.item(i);

           if(rgpSubNode.getNodeName().equals(rgpPrefix + ":rgpStatus")) {
                 string type;
                 getNodeData(rgpSubNode.getAttributes().getNamedItem("s"),type);
                 m_rgp_statuses.push_back(type);
           }
    }

}
