/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 Afilias
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include "domtools/dom_output.h"
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "SecDNSInfData.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string SecDNSInfData::toXML()
{
    return "Nothing to do";
}
     
void SecDNSInfData::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered SecDNSInfData::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }

    DOM_Node secDNSNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("secDNS:infData").item(0);

    if (secDNSNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no secDNS extension to parse");
    }

    DOM_NodeList secDNSNodes = secDNSNode.getChildNodes();

    if(secDNSNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty secDNS extension");
    }

    m_ds_data.ref(new SecDNSDsData_seq());

    DOMString secDNSPrefix = secDNSNode.getPrefix();
    for(unsigned int i = 0; i < secDNSNodes.getLength(); i++)
        {
                DOM_Node secDNSSubNode = secDNSNodes.item(i);

                if(secDNSSubNode.getNodeName().equals(secDNSPrefix + ":dsData")) {
                     DOM_NodeList secDNSChildNode = secDNSSubNode.getChildNodes();

                     SecDNSDsData_ref temp_ds_data(new SecDNSDsData());
                     for(unsigned int j = 0;j < secDNSChildNode.getLength();j++) {
                         DOM_Node secDNSChildSubNode = secDNSChildNode.item(j);

                         if (secDNSChildSubNode.getNodeName().equals(secDNSPrefix + ":keyTag")) {
                            string value;
                            getNodeData(secDNSChildSubNode, value);
                            temp_ds_data->m_ds_key_tag.ref(new epp_long(atol(value.c_str())));
                         }
                         if (secDNSChildSubNode.getNodeName().equals(secDNSPrefix + ":alg")) {
                            string value;
                            getNodeData(secDNSChildSubNode, value);
                            temp_ds_data->m_ds_alg.ref(new epp_short((short)atoi(value.c_str())));
                         }
                         if (secDNSChildSubNode.getNodeName().equals(secDNSPrefix + ":digestType")) {
                            string value;
                            getNodeData(secDNSChildSubNode, value);
                            temp_ds_data->m_ds_digest_type.ref(new epp_short((short)atoi(value.c_str())));
                         }
                         if (secDNSChildSubNode.getNodeName().equals(secDNSPrefix + ":digest")) {
                            string value;
                            getNodeData(secDNSChildSubNode, value);
                            temp_ds_data->m_ds_digest.ref(new epp_string(value));
                         }
                         if (secDNSChildSubNode.getNodeName().equals(secDNSPrefix + ":maxSigLife")) {
                            string value;
                            getNodeData(secDNSChildSubNode, value);
                            temp_ds_data->m_ds_max_sig_life.ref(new epp_long(atol(value.c_str())));
                         }
                         if (secDNSChildSubNode.getNodeName().equals(secDNSPrefix + ":keyData")) {
                            DOM_NodeList secDNSChildNode_2 = secDNSChildSubNode.getChildNodes();

                            temp_ds_data->m_ds_key_data.ref(new SecDNSKeyData());
                            for(unsigned int k = 0;k < secDNSChildNode_2.getLength();k++) {
                                DOM_Node secDNSChildSubNode_2 = secDNSChildNode_2.item(k);

                                if (secDNSChildSubNode_2.getNodeName().equals(secDNSPrefix + ":flags")) {
                                    string value;
                                    getNodeData(secDNSChildSubNode_2, value);
                                    temp_ds_data->m_ds_key_data->m_ds_flags.ref(new epp_short((short)atoi(value.c_str())));
                                }                     
                                if (secDNSChildSubNode_2.getNodeName().equals(secDNSPrefix + ":protocol")) {
                                    string value;
                                    getNodeData(secDNSChildSubNode_2, value);
                                    temp_ds_data->m_ds_key_data->m_ds_protocol.ref(new epp_short((short)atoi(value.c_str())));
                                }
                                if (secDNSChildSubNode_2.getNodeName().equals(secDNSPrefix + ":alg")) {
                                    string value;
                                    getNodeData(secDNSChildSubNode_2, value);
                                    temp_ds_data->m_ds_key_data->m_ds_alg.ref(new epp_short((short)atoi(value.c_str())));
                                }
                                if (secDNSChildSubNode_2.getNodeName().equals(secDNSPrefix + ":pubKey")) {
                                    string value;
                                    getNodeData(secDNSChildSubNode_2, value);
                                    temp_ds_data->m_ds_key_data->m_ds_pub_key.ref(new epp_string(value));
                                }
                            }
                         }
        
                    }
                    m_ds_data->push_back(*temp_ds_data);
               }
      }

      return;
}
