/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
* Copyright (C) 2003 LibertyRMS
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
************************************************************************/

#include <memory>
#include "domtools/dom_wrapper.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_eppXMLbase.h"
#include "epp-rtk-cpp/config.h"
#include "Trademark.h"

using namespace std;
using namespace domtools;
using namespace eppobject::domain;
using namespace eppobject::epp;

eppobject::epp::epp_string Trademark::toXML()
{
#ifdef RTK_DEBUG
    cout << "Trademark::toXML(): entered ["<<*m_command<<"] ["<<*m_tld<<"]"<<endl;
#endif

    if (m_command == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing trademark command value"); 
    }

    if (m_tld == NULL ) {
        throw epp_XMLException(__FILE__, __LINE__, "Missing trademark tld value"); 
    }

    EPP_output xml_request;
    xml_request.setWhitespace(Whitespace);

    string trademark_tag = "trademark:";
    string tld_specific;

    if (*m_tld == "mobi") {
        tld_specific = "mobi-trademark-1.0";
    }
    else if (*m_tld == "info") {
        tld_specific = "info-trademark-1.0";
    }
    else if (*m_tld == "in") {
        tld_specific = "in-trademark-1.0";
    }
    else {
        throw epp_XMLException(__FILE__, __LINE__, "unsupported tld");
    }

    trademark_tag.append(*m_command);
    
    xml_request.beginTag(trademark_tag);
    xml_request.putAttribute("xmlns:trademark", "urn:afilias:params:xml:ns:ext:" + tld_specific);
    xml_request.putAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    xml_request.putAttribute("xsi:schemaLocation", "urn:afilias:params:xml:ns:ext:" + tld_specific + " " + tld_specific + ".xsd");

    if (m_trademark_data == NULL) {
        throw epp_XMLException(__FILE__, __LINE__, "trademark data need");
    }

    if (*m_trademark_data->m_remove) {
        xml_request.beginTag("trademark:rem");
        xml_request.endTag("trademark:rem");
    }
    else {
        if (m_trademark_data->m_name != NULL) {
            xml_request.beginTag("trademark:name");
            xml_request.putCDATA(*m_trademark_data->m_name);
            xml_request.endTag("trademark:name");
        }

        if (m_trademark_data->m_number != NULL) {
            xml_request.beginTag("trademark:number");
            xml_request.putCDATA(*m_trademark_data->m_number);
            xml_request.endTag("trademark:number");
        }

        if (m_trademark_data->m_country != NULL) {
            xml_request.beginTag("trademark:country");
            xml_request.putCDATA(*m_trademark_data->m_country);
            xml_request.endTag("trademark:country");
        }

        if (m_trademark_data->m_date != NULL) {
            xml_request.beginTag("trademark:date");
            xml_request.putCDATA(*m_trademark_data->m_date);
            xml_request.endTag("trademark:date");
        }

        if (m_trademark_data->m_reg_date != NULL) {
            xml_request.beginTag("trademark:regDate");
            xml_request.putCDATA(*m_trademark_data->m_reg_date);
            xml_request.endTag("trademark:regDate");
        }

        if (m_trademark_data->m_app_date != NULL) {
            xml_request.beginTag("trademark:appDate");
            xml_request.putCDATA(*m_trademark_data->m_app_date);
            xml_request.endTag("trademark:appDate");
        }
   
        if (m_trademark_data->m_owner_country != NULL) {
            xml_request.beginTag("trademark:ownerCountry");
            xml_request.putCDATA(*m_trademark_data->m_owner_country);
            xml_request.endTag("trademark:ownerCountry");
        }
    }

    xml_request.endTag(trademark_tag);

#ifdef RTK_DEBUG
    cout << "Trademark::toXML(): leaving ["<<xml_request.getString()<<"]"<<endl;
#endif
    return xml_request.getString();
}
     
void Trademark::fromXML(const eppobject::epp::epp_string & xml)
{
#ifdef RTK_DEBUG
    cerr << "entered Trademark::fromXML() ["<<xml<<"]"<<endl;
#endif

    DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+xml+"</parseme>");

    DOM_Node parsemeNode = getSingleTag(doc, "parseme");
    if(parsemeNode.isNull())
    {
        throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
    }
     
    DOM_Node trademarkNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("trademark:infData").item(0);
    
    if (trademarkNode.isNull()) {
        throw epp_XMLException(__FILE__, __LINE__, "Empty parsme -- no trademark extension to parse");
    }
     
    DOM_NodeList trademarkNodes = trademarkNode.getChildNodes();
   
    if(trademarkNodes.getLength() == 0)
    {
        throw epp_XMLException(__FILE__, __LINE__, "Empty Trademark extension");
    }

    m_trademark_data.ref(new TrademarkData());

    DOMString trademarkPrefix = trademarkNode.getPrefix();

    for(unsigned int i = 0; i < trademarkNodes.getLength(); i++)
	{
		DOM_Node trademarkSubNode = trademarkNodes.item(i);

		if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":name")) {
			string node_data;
			getNodeData(trademarkSubNode, node_data);
			m_trademark_data->m_name.ref(new epp_string(node_data));
		} 
		if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":number")) {
			string node_data;
			getNodeData(trademarkSubNode, node_data);
			m_trademark_data->m_number.ref(new epp_string(node_data));
		} 
		if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":country")) {
			string node_data;
			getNodeData(trademarkSubNode, node_data);
			m_trademark_data->m_country.ref(new epp_string(node_data));
		} 
		if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":date")) {
			string node_data;
			getNodeData(trademarkSubNode, node_data);
			m_trademark_data->m_date.ref(new epp_string(node_data));
		} 
		if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":appDate")) {
			string node_data;
			getNodeData(trademarkSubNode, node_data);
			m_trademark_data->m_app_date.ref(new epp_string(node_data));
		} 
		if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":regDate")) {
			string node_data;
			getNodeData(trademarkSubNode, node_data);
			m_trademark_data->m_reg_date.ref(new epp_string(node_data));
		}
                if(trademarkSubNode.getNodeName().equals(trademarkPrefix + ":ownerCountry")) {
                        string node_data;
                        getNodeData(trademarkSubNode, node_data);
                        m_trademark_data->m_owner_country.ref(new epp_string(node_data));
                }
              
	}
}
