/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/liberty-rtk-addon/c++/org/src/main_aero_domainexample.cc,v 1.2 2006/08/23 20:31:49 ewang2004 Exp $ */

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_DomainCreateReq.h"
#include "epp-rtk-cpp/data/epp_DomainCreateRsp.h"
#include "epp-rtk-cpp/data/epp_DomainUpdateReq.h"
#include "epp-rtk-cpp/data/epp_DomainUpdateRsp.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"
#include "epp-rtk-cpp/epp_DomainUpdate.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "epp-rtk-cpp/data/epp_DomainInfoReq.h"
#include "epp-rtk-cpp/data/epp_DomainInfoRsp.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "AeroDomain.h"
#include "EnsInfo.h"

#include "epp-rtk-cpp/epp_ContactCheck.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCreateRsp_ref & req)
{
	cout << "Printing Response:\n_______________________________\n";

	if(req != NULL) {

		printResponsePart1(req->m_rsp);
		printResDataHeaders("domain", "creData");

		if(req->m_name != NULL)
			cerr << "<domain:name>" << *req-> m_name << "</domain:name>" << endl;

		if(req->m_creation_date != NULL)
			cerr << "<domain:crDate>" << *req-> m_creation_date << "</domain:crDate>" << endl;

		if(req->m_expiration_date != NULL)
			cerr << "<domain:exDate>" << *req-> m_expiration_date << "</domain:exDate>" << endl;

		printResDataFooter("domain", "creData");
		printResponsePart2(req->m_rsp);
	}


	cout << "_______________________________\n";
} // printRsp

int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;

	string extra_usage = "[-extradata=trid=xxx:domain=abc.tld:period=1:ns1=asdf:ns2=asdf{:...}:reg=abc:contact1=t/abc{:...}:authinfo=pw/38239]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

	try {

        /**************************************************************/
        /**************    normal domain creatioin       **************/
        /**************************************************************/
		epp_DomainCreateReq_ref l_req(new epp_DomainCreateReq());

		string trID = "ABC-12345";
		if(extraDataMap.find("trid") != extraDataMap.end()) {
			trID = extraDataMap["trid"];
		}

		if ( extraDataMap.find("ens_id") == extraDataMap.end() ||
				extraDataMap.find("ens_key") == extraDataMap.end() ) {
			cerr << "please provide ens_id and ens_key by: [-extradata=ens_id=id:ens_key=key]" << endl;

			return(1);
		}

		AeroDomain_ref aero_domain_extension(new AeroDomain());
		aero_domain_extension->m_command.ref(new epp_string("create"));
		aero_domain_extension->m_ens_auth_id.ref(new epp_string(extraDataMap["ens_id"]));
		aero_domain_extension->m_ens_auth_key.ref(new epp_string(extraDataMap["ens_key"]));

		l_req->m_cmd.ref(new epp_Command(NULL,
					aero_domain_extension,
					epp_trid(trID)));

		if ( extraDataMap.find("domain") == extraDataMap.end() ) {
			l_req->m_name.ref(new epp_string("example.aero"));
		}
		else {
			l_req->m_name.ref(new epp_string(extraDataMap["domain"]));
		}

		l_req->m_period.ref(new epp_DomainPeriod(YEAR, 2));

		// we don't care about name servers

		if ( extraDataMap.find("reg") == extraDataMap.end() ) {
			l_req->m_registrant.ref(new epp_string("ClientY001"));
		}
		else {
			l_req->m_registrant.ref(new epp_string(extraDataMap["reg"]));
		}

		l_req->m_contacts.ref(new epp_domain_contact_seq());

		string contact_id = "ClientY002";

		if ( extraDataMap.find("contact") != extraDataMap.end() ) {
			contact_id = extraDataMap["contact"];
		}

		epp_DomainContact Cont1(ADMIN, epp_string(contact_id));
		epp_DomainContact Cont2(TECH,  epp_string(contact_id));
		l_req->m_contacts->push_back(Cont1);
		l_req->m_contacts->push_back(Cont2);

		l_req->m_auth_info.ref(new epp_AuthInfo());
		l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
		l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

		epp_DomainCreate_ref l_obj(new epp_DomainCreate());

		l_obj->setRequestData(*l_req);

		test.processAction(l_obj);

		epp_DomainCreateRsp_ref l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
		printRsp(l_rsp);
#endif
		epp_DomainInfoReq_ref domain_info_req(new epp_DomainInfoReq());

		domain_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
					NULL,
					eppobject::epp::epp_trid(trID)));

		if(extraDataMap.find("domain") == extraDataMap.end()) {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string("example.aero"));
		}
		else {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["domain"]));
		}

		domain_info_req->m_hosts_type.ref(new epp_DomainHostsType(eppobject::domain::ALL));

		epp_DomainInfo_ref domain_info_obj(new epp_DomainInfo());

		domain_info_obj->setRequestData(*domain_info_req);

		test.processAction(domain_info_obj);

		epp_DomainInfoRsp_ref domain_info_rsp = domain_info_obj->getResponseData();
		if(domain_info_rsp->m_rsp->m_ext_strings != NULL) {
			cout << "==== Domain Info Response Extenstion ===== \n";
			AeroDomain_ref extension(new AeroDomain());
			eppobject::epp::epp_xml_string_seq_ref extension_strings;
			extension_strings = domain_info_rsp->m_rsp->m_ext_strings;
			eppobject::epp::epp_xml_string first_extension = extension_strings->front();

			extension->fromXML(first_extension);
			cout << "aero:ensAuthID - \"" << *extension->m_ens_auth_id << "\"\n";
		}

        /**************************************************************/
        /**************    pending domain creatioin      **************/
        /**************************************************************/
		epp_DomainCreateReq_ref l_req2(new epp_DomainCreateReq());

		AeroDomain_ref aero_domain_extension2(new AeroDomain());
		aero_domain_extension2->m_command.ref(new epp_string("create"));

        EnsInfo_ref ens_info(new EnsInfo());
        ens_info->m_ens_class_.ref(new epp_string_seq());
        ens_info->m_ens_class_->push_back("airport");
        ens_info->m_ens_class_->push_back("airline");
        ens_info->m_registrantGroup.ref(new epp_string("airport"));
        ens_info->m_ensO.ref(new epp_string("sita"));
        ens_info->m_requestType.ref(new epp_string("manual"));
        ens_info->m_registrationType.ref(new epp_string("ADA ADB BXN DLM ESB ISE IST NAV TZX"));
        ens_info->m_credentialsType.ref(new epp_string("credentials type"));
        ens_info->m_credentialsValue.ref(new epp_string("credentials value"));
        ens_info->m_codeValue.ref(new epp_string("code value"));
        ens_info->m_uniqueIdentifier.ref(new epp_string("XNVS-123455"));

		aero_domain_extension2->m_ens_info = ens_info;

		l_req2->m_cmd.ref(new epp_Command(NULL,
					aero_domain_extension2,
					epp_trid("ABC-123456")));

        string domain_name("rtk-pending-create-");
        srand((unsigned)time(0));
        int random_integer = 1+(int) (100.0*rand()/(RAND_MAX+1.0));
        char rand_int[3]; 
        sprintf(rand_int, "%d", random_integer);
        domain_name.append(rand_int);
        domain_name.append(".aero");

        epp_string_ref pending_domain(new epp_string(domain_name));

		l_req2->m_name = (pending_domain);

		l_req2->m_period.ref(new epp_DomainPeriod(YEAR, 2));

		// we don't care about name servers
        l_req2->m_registrant.ref(new epp_string("ClientY001"));

		l_req2->m_contacts.ref(new epp_domain_contact_seq());

		l_req2->m_contacts->push_back(epp_DomainContact(ADMIN, epp_string("ClientY002")));
		l_req2->m_contacts->push_back(epp_DomainContact(TECH, epp_string("ClientY002")));

		l_req2->m_auth_info.ref(new epp_AuthInfo());
		l_req2->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
		l_req2->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

		epp_DomainCreate_ref l_obj2(new epp_DomainCreate());

		l_obj2->setRequestData(*l_req2);

		test.processAction(l_obj2);

		epp_DomainCreateRsp_ref l_rsp2 = l_obj2->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
		printRsp(l_rsp2);
#endif
		epp_DomainInfoReq_ref domain_info_req2(new epp_DomainInfoReq());

		domain_info_req2->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
					NULL,
					eppobject::epp::epp_trid("ABC-123456")));

        domain_info_req2->m_name = pending_domain;

		domain_info_req2->m_hosts_type.ref(new epp_DomainHostsType(eppobject::domain::ALL));

		epp_DomainInfo_ref domain_info_obj2(new epp_DomainInfo());

		domain_info_obj2->setRequestData(*domain_info_req2);

		test.processAction(domain_info_obj2);

		epp_DomainInfoRsp_ref domain_info_rsp2 = domain_info_obj2->getResponseData();
		if(domain_info_rsp2->m_rsp->m_ext_strings != NULL) {
			cout << "==== Domain Info Response Extenstion ===== \n";
			AeroDomain_ref extension2(new AeroDomain());
			eppobject::epp::epp_xml_string_seq_ref extension_strings2;
			extension_strings2 = domain_info_rsp2->m_rsp->m_ext_strings;
			eppobject::epp::epp_xml_string first_extension2 = extension_strings2->front();

			extension2->fromXML(first_extension2);
			cout << "aero:registrantGroup - \"" << *extension2->m_ens_info->m_registrantGroup << "\"\n";
			cout << "aero:ensO - \"" << *extension2->m_ens_info->m_ensO << "\"\n";
			cout << "aero:requestType - \"" << *extension2->m_ens_info->m_requestType << "\"\n";
			cout << "aero:registrationType - \"" << *extension2->m_ens_info->m_registrationType << "\"\n";
			cout << "aero:credentialsType - \"" << *extension2->m_ens_info->m_credentialsType << "\"\n";
			cout << "aero:credentialsValue - \"" << *extension2->m_ens_info->m_credentialsValue << "\"\n";
			cout << "aero:codeValue - \"" << *extension2->m_ens_info->m_codeValue << "\"\n";
			cout << "aero:uniqueIdentifier - \"" << *extension2->m_ens_info->m_uniqueIdentifier << "\"\n";
		}

        /**************************************************************/
        /**************    update domain ens contact      *************/
        /**************************************************************/
        if ( extraDataMap.find("ens_id2") == extraDataMap.end() ||
                extraDataMap.find("ens_key2") == extraDataMap.end() ) {
            cerr << "please provide another ens_id/ens_key pair by: [-extradata=ens_id2=id:ens_key2=key]" << endl;
                                                                                                                             
            return(1);
        }

        epp_DomainUpdateReq_ref l_req3(new epp_DomainUpdateReq());
        l_req3->m_name.ref(new epp_string(extraDataMap["domain"]));
        l_req3->m_change.ref(new epp_DomainUpdateChange());

        AeroDomain_ref aero_domain_extension3(new AeroDomain());
        aero_domain_extension3->m_command.ref(new epp_string("update"));
        aero_domain_extension3->m_ens_auth_id.ref(new epp_string(extraDataMap["ens_id2"]));
        aero_domain_extension3->m_ens_auth_key.ref(new epp_string(extraDataMap["ens_key2"]));
                                                                                                                             
        l_req3->m_cmd.ref(new epp_Command(NULL,
                    aero_domain_extension3,
                    epp_trid("ABC-12347")));

        epp_DomainUpdate_ref l_obj3(new epp_DomainUpdate());

        l_obj3->setRequestData(*l_req3);

        test.processAction(l_obj3);

        return 0;
    }
    catch(const epp_XMLException & ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(const epp_Exception & ex) {
        cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
        cerr << "<response>" << endl;
        printResultsSeq(ex.m_details);
        printTransID(ex.m_trans_id);
        cerr << "</response>" << endl;
#endif
    }
    catch(const epptransport::epp_TrException & ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(...) {
        cout << "Something else went wrong...\n";
    }

}
