/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/liberty-rtk-addon/c++/org/src/main_aero_enscontactexample.cc,v 1.2 2006/08/23 20:31:49 ewang2004 Exp $ */

#include <string>
#include <vector>
#include <iostream>
#include <memory>

//#include "epp-rtk-cpp/data/epp_ContactInfoReq.h"
//#include "epp-rtk-cpp/data/epp_ContactInfoRsp.h"
#include "epp-rtk-cpp/epp_ContactInfo.h"
//#include "epp-rtk-cpp/data/epp_ContactCheckReq.h"
//#include "epp-rtk-cpp/data/epp_ContactCheckRsp.h"
#include "epp-rtk-cpp/epp_ContactCheck.h"
#include "epp-rtk-cpp/epp_ContactCreate.h"
#include "epp-rtk-cpp/epp_ContactUpdate.h"
//#include "epp-rtk-cpp/data/epp_contactXMLbase.h"
#include "epp-rtk-cpp/data/epp_ContactCreateReq.h"
#include "epp-rtk-cpp/data/epp_ContactCreateRsp.h"
#include "epp-rtk-cpp/data/epp_ContactUpdateReq.h"
#include "epp-rtk-cpp/data/epp_ContactUpdateRsp.h"
//#include "epp-rtk-cpp/data/epp_DomainContact.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"

#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_AuthInfo.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"

#include "AeroContactN.h"

using namespace std;
using namespace eppobject::contact;
using namespace eppobject::epp;

void printRsp(const epp_ContactUpdateRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
                                                                                                                             
   if(req != NULL)
     printResponsePart(req->m_rsp);
                                                                                                                             
   cout << "_______________________________\n";
} // printRsp

void printRsp(const epp_ContactCreateRsp_ref & req)
{
    cout << "Printing Response:\n_______________________________\n";

    if(req != NULL) {

        printResponsePart1(req->m_rsp);

        printResDataHeaders("contact", "creData");

        if(req->m_id != NULL)
            cerr << "<contact:id>" << *req-> m_id << "</contact:id>" << endl;

        if(req->m_creation_date != NULL)
            cerr << "<contact:crDate>" << *req-> m_creation_date << "</contact:crDate>" << endl;

        printResDataFooter("contact", "creData");
        printResponsePart2(req->m_rsp);
    }


    cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;

	string extra_usage = "[-extradata=trid=xxx:id=abc]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

    try {

        /**************************************************************
         *		ENS Contact Info
         *************************************************************/

        epp_ContactInfoReq_ref l_req(new epp_ContactInfoReq());

        string trID = "ABC-12345";
        if(extraDataMap.find("trid") != extraDataMap.end()) trID = extraDataMap["trid"];

        l_req->m_cmd.ref(new epp_Command(NULL,
                    epp_trid(trID)));

        if(extraDataMap.find("id") == extraDataMap.end()) {
            l_req->m_id.ref(new epp_string("aens11"));
        }
        else {
            l_req->m_id.ref(new epp_string(extraDataMap["id"]));
        }

        l_req->m_auth_info.ref(new epp_AuthInfo());
        l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));

        if(extraDataMap.find("key") == extraDataMap.end()) {
            l_req->m_auth_info->m_value.ref(new epp_string("111111"));
        }
        else {
            l_req->m_auth_info->m_value.ref(new epp_string(extraDataMap["key"]));
        }

        epp_ContactInfo_ref l_obj(new epp_ContactInfo());

        l_obj->setRequestData(*l_req);

        test.processAction(l_obj);

        epp_ContactInfoRsp_ref l_rsp = l_obj->getResponseData();

        if(l_rsp->m_rsp->m_ext_strings != NULL) {
            cout << "==== ENS Contact Info Response Extenstion ===== \n";
            AeroContactN_ref extension(new AeroContactN());
            eppobject::epp::epp_xml_string_seq_ref extension_strings;
            extension_strings = l_rsp->m_rsp->m_ext_strings;
            eppobject::epp::epp_xml_string first_extension = extension_strings->front();

            extension->fromXML(first_extension);
            cout << "aero:registrantGroup - \"" << *extension->m_ens_info->m_registrantGroup << "\"\n";
            cout << "aero:ensO - \"" << *extension->m_ens_info->m_ensO << "\"\n";
            cout << "aero:requestType - \"" << *extension->m_ens_info->m_requestType << "\"\n";
            cout << "aero:registrationType - \"" << *extension->m_ens_info->m_registrationType << "\"\n";
            cout << "aero:credentialsType - \"" << *extension->m_ens_info->m_credentialsType << "\"\n";
            cout << "aero:credentialsValue - \"" << *extension->m_ens_info->m_credentialsValue << "\"\n";
            cout << "aero:codeValue - \"" << *extension->m_ens_info->m_codeValue << "\"\n";
            cout << "aero:uniqueIdentifier - \"" << *extension->m_ens_info->m_uniqueIdentifier << "\"\n";
        }
        else {
            cout << "==== No Response Extenstion for ENS Contact Info ===== \n";
        }

        /**************************************************************
         *		create Contact with maintainer url                    *
         *************************************************************/

        epp_ContactCreateReq_ref l_req2(new epp_ContactCreateReq());

        AeroContactN_ref aero_contact_extension(new AeroContactN());
        aero_contact_extension->m_command.ref(new epp_string("create"));
        aero_contact_extension->m_maintainer_url.ref(new epp_string("http://afilias-test-1.aero"));

        l_req2->m_cmd.ref(new epp_Command(NULL,
                    aero_contact_extension,
                    epp_trid(trID)));

        if( extraDataMap.find("contact") == extraDataMap.end() ) {
            l_req2->m_id.ref(new epp_string("sh8013"));

            epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());
            addr->m_name.ref(new epp_string("John Doe"));
            addr->m_org.ref(new epp_string("Example Inc."));

            addr->m_address.ref(new epp_ContactAddress());
            addr->m_address->m_street1.ref(new epp_string("123 Example Dr."));
            addr->m_address->m_street2.ref(new epp_string("Suite 100"));
            addr->m_address->m_city.ref(new epp_string("Dulles"));
            addr->m_address->m_state_province.ref(new epp_string("VA"));
            addr->m_address->m_postal_code.ref(new epp_string("20166-6503"));
            addr->m_address->m_country_code.ref(new epp_string("US"));

            addr->m_type.ref(new epp_ContactPostalInfoType(INT));
            l_req2->m_addresses.ref(new epp_ContactNameAddress_seq());
            l_req2->m_addresses->push_back(*addr);

            l_req2->m_voice.ref(new epp_ContactPhone(epp_string("1234"),
                        epp_string("+1.7035555555")));
            l_req2->m_fax.ref(new epp_ContactPhone());
            l_req2->m_fax->m_value.ref(new epp_string("+1.7035555556"));

            l_req2->m_email.ref(new epp_string("jdoe@example.com"));
            l_req2->m_auth_info.ref(new epp_AuthInfo());
            l_req2->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
            l_req2->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

        } else {

            string contactDataStr = extraDataMap["contact"];
            vector<string> contactData;

            unsigned int end;
            while((end = contactDataStr.find('/')) != string::npos) {

                string tmp = contactDataStr.substr(0,end);
#ifdef RTK_DEBUG
                cout << "The contact data string token ["<< tmp <<"]" << endl;
#endif
                contactData.push_back(tmp);

                contactDataStr.erase(0,end+1);

            } // while

            contactData.push_back(contactDataStr);
            while ( contactData.size() < 16 ) contactData.push_back("");


            cout << "building the contact...\n";
            if(contactData[0] != "-1") l_req2->m_id.ref(new epp_string(contactData[0]));

            if(contactData[1] != "-1") {
                l_req2->m_auth_info.ref(new epp_AuthInfo());
                l_req2->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
                l_req2->m_auth_info->m_value.ref(new epp_string(contactData[1]));
            }

            epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());
            if(contactData[2] != "") addr->m_name.ref(new epp_string(contactData[2]));
            if(contactData[3] != "") addr->m_org.ref(new epp_string(contactData[3]));

            addr->m_address.ref(new epp_ContactAddress());
            if(contactData[4] != "-1") addr->m_address->m_street1.ref(new epp_string(contactData[4]));
            if(contactData[5] != "-1") addr->m_address->m_street2.ref(new epp_string(contactData[5]));
            if(contactData[6] != "-1") addr->m_address->m_street3.ref(new epp_string(contactData[6]));
            if(contactData[7] != "-1") addr->m_address->m_city.ref(new epp_string(contactData[7]));
            if(contactData[8] != "-1") addr->m_address->m_state_province.ref(new epp_string(contactData[8]));
            if(contactData[9] != "-1") addr->m_address->m_country_code.ref(new epp_string(contactData[9]));
            if(contactData[10] != "-1") addr->m_address->m_postal_code.ref(new epp_string(contactData[10]));

            addr->m_type.ref(new epp_ContactPostalInfoType(INT));
            l_req2->m_addresses.ref(new epp_ContactNameAddress_seq());
            l_req2->m_addresses->push_back(*addr);

            if(contactData[11] != "-1" || contactData[12] != "-1") {
                l_req2->m_voice.ref(new epp_ContactPhone());
                if(contactData[11] != "-1") l_req2->m_voice->m_value.ref(new epp_string(contactData[11]));
                if(contactData[12] != "-1") l_req2->m_voice->m_extension.ref(new epp_string(contactData[12]));
            }

            if(contactData[13] != "-1") l_req2->m_email.ref(new epp_string(contactData[13]));

            if(contactData[14] != "-1" || contactData[15] != "-1") {
                l_req2->m_fax.ref(new epp_ContactPhone());
                if(contactData[14] != "-1") l_req2->m_fax->m_value.ref(new epp_string(contactData[14]));
                if(contactData[15] != "-1") l_req2->m_fax->m_extension.ref(new epp_string(contactData[15]));
            }

        }      

        cout << "Creating the contact create request...\n";

        epp_ContactCreate_ref l_obj2(new epp_ContactCreate());

        l_obj2->setRequestData(*l_req2);

        test.processAction(l_obj2);

        epp_ContactCreateRsp_ref l_rsp2 = l_obj2->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
        printRsp(l_rsp2);
#endif 

        /**************************************************************
         *		     update Contact maintainer url                    *
         *************************************************************/

        epp_ContactUpdateReq_ref l_req3(new epp_ContactUpdateReq());

        if ( extraDataMap.find("contact") == extraDataMap.end() ) l_req3->m_id.ref(new epp_string("sh8013"));
        else l_req3->m_id.ref(new epp_string(extraDataMap["contact"]));

        epp_ContactUpdateChange_ref chg(new epp_ContactUpdateChange());
        l_req3->m_change = chg;

        AeroContactN_ref aero_contact_extension3(new AeroContactN());
        aero_contact_extension3->m_command.ref(new epp_string("update"));
        aero_contact_extension3->m_maintainer_url.ref(new epp_string("http://afilias-test-2.aero"));

        l_req3->m_cmd.ref(new epp_Command(NULL,
                    aero_contact_extension3,
                    epp_trid(trID)));

        epp_ContactUpdate_ref l_obj3(new epp_ContactUpdate());

        l_obj3->setRequestData(*l_req3);                                                                                                                              
        test.processAction(l_obj3);

        epp_ContactUpdateRsp_ref l_rsp3 = l_obj3->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
        printRsp(l_rsp3);
#endif
        return 0;

    }
    catch(const epp_XMLException & ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(const epp_Exception & ex) {
        cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
        cerr << "<response>" << endl;
        printResultsSeq(ex.m_details);
        printTransID(ex.m_trans_id);
        cerr << "</response>" << endl;
#endif
    }
    catch(const epptransport::epp_TrException & ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(...) {
        cout << "Something else went wrong...\n";
    }

    return 0;
}
