#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_ContactCreateReq.h"
#include "epp-rtk-cpp/data/epp_ContactCreateRsp.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_ContactCreate.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "epp-rtk-cpp/data/epp_ContactInfoReq.h"
#include "epp-rtk-cpp/data/epp_ContactInfoRsp.h"
#include "epp-rtk-cpp/data/epp_contactXMLbase.h"
#include "epp-rtk-cpp/data/epp_ContactUpdateReq.h"
#include "epp-rtk-cpp/data/epp_ContactUpdateRsp.h"
#include "epp-rtk-cpp/epp_ContactUpdate.h"
#include "epp-rtk-cpp/epp_ContactInfo.h"
#include "asiadata.h"
#include "AsiaCEDContact.h"
#include "AsiaCEDContactData.h"

#include "epp-rtk-cpp/epp_ContactCheck.h"

using namespace std;
using namespace eppobject::contact;
using namespace eppobject::epp;

void printRsp(const epp_ContactCreateRsp_ref & req)
{
       cout << "Printing Response:\n_______________________________\n";

   if(req != NULL) {

      printResponsePart1(req->m_rsp);

      printResDataHeaders("contact", "creData");

      if(req->m_id != NULL)
        cerr << "<contact:id>" << *req-> m_id << "</contact:id>" << endl;

      if(req->m_creation_date != NULL)
        cerr << "<contact:crDate>" << *req-> m_creation_date << "</contact:crDate>" << endl;

      printResDataFooter("contact", "creData");
      printResponsePart2(req->m_rsp);
   }


   cout << "_______________________________\n";

} // printRsp

void printRsp(const epp_ContactUpdateRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";

   if(req != NULL) {

      printResponsePart(req->m_rsp);
   }


   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;

	string extra_usage = "[-extradata=trid=xxx:contact_id=abc:cclocality=cn:localitysp=abc:localitycity=abc:legalentitytype=abc:identform=abc:identnumber=12345:otherletype=abc:otheridentform=abc]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

	try {

		epp_ContactCreateReq_ref l_req(new epp_ContactCreateReq());

		string trID = "ABC-12345";
		if(extraDataMap.find("trid") != extraDataMap.end()) {
			trID = extraDataMap["trid"];
		}

                epp_ContactNameAddress_ref addr(new epp_ContactNameAddress());
                addr->m_name.ref(new epp_string("John Doe"));
                addr->m_org.ref(new epp_string("Example Inc."));

                addr->m_address.ref(new epp_ContactAddress());
                addr->m_address->m_street1.ref(new epp_string("123 Example Dr."));
                addr->m_address->m_street2.ref(new epp_string("Suite 100"));
                addr->m_address->m_city.ref(new epp_string("Dulles"));
                addr->m_address->m_state_province.ref(new epp_string("VA"));
                addr->m_address->m_postal_code.ref(new epp_string("20166-6503"));
                addr->m_address->m_country_code.ref(new epp_string("US"));

                addr->m_type.ref(new epp_ContactPostalInfoType(INT));
                l_req->m_addresses.ref(new epp_ContactNameAddress_seq());
                l_req->m_addresses->push_back(*addr);

                l_req->m_voice.ref(new epp_ContactPhone(epp_string("1234"),
                                                 epp_string("+1.7035555555")));
                l_req->m_fax.ref(new epp_ContactPhone());
                l_req->m_fax->m_value.ref(new epp_string("+1.7035555556"));

                l_req->m_email.ref(new epp_string("jdoe@example.com"));
                l_req->m_auth_info.ref(new epp_AuthInfo());
                l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
                l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

                // Extension

		AsiaCEDContact_ref asia_contact_extension(new AsiaCEDContact());
		asia_contact_extension->m_command.ref(new epp_string("create"));
                asia_contact_extension->m_ced_contact_data.ref(new AsiaCEDContactData());

                if ( extraDataMap.find("cclocality") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_ccLocality.ref(new epp_string("cn"));
                }
                else {
                     asia_contact_extension->m_ced_contact_data->m_ccLocality.ref(new epp_string(extraDataMap["cclocality"]));
                }
                if ( extraDataMap.find("localitysp") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_localitySp.ref(new epp_string("ontario"));
                } else {
                     asia_contact_extension->m_ced_contact_data->m_localitySp.ref(new epp_string(extraDataMap["localitysp"]));
                } 
                if ( extraDataMap.find("localitycity") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_localityCity.ref(new epp_string("toronto"));
                } else {
                     asia_contact_extension->m_ced_contact_data->m_localityCity.ref(new epp_string(extraDataMap["localitycity"]));
                }
                if ( extraDataMap.find("legalentitytype") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_legalEntityType.ref(new epp_string("naturalPerson"));
                } else {
                     asia_contact_extension->m_ced_contact_data->m_legalEntityType.ref(new epp_string(extraDataMap["legalentitytype"]));
                }
                if ( extraDataMap.find("identform") == extraDataMap.end() ) {  
                     asia_contact_extension->m_ced_contact_data->m_identForm.ref(new epp_string("passport"));
                } else {
                     asia_contact_extension->m_ced_contact_data->m_identForm.ref(new epp_string(extraDataMap["identform"]));
                }
                if ( extraDataMap.find("identnumber") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_identNumber.ref(new epp_string("12345678"));
                } else {
                     asia_contact_extension->m_ced_contact_data->m_identNumber.ref(new epp_string(extraDataMap["identnumber"]));
                }
                if ( extraDataMap.find("otherletype") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_otherLEType.ref(new epp_string("naturalPerson"));
                } else {
                     asia_contact_extension->m_ced_contact_data->m_otherLEType.ref(new epp_string(extraDataMap["otherletype"]));
                }
                if ( extraDataMap.find("otheridentform") == extraDataMap.end() ) {
                     asia_contact_extension->m_ced_contact_data->m_otherIdentForm.ref(new epp_string("certificate"));                } else {
                     asia_contact_extension->m_ced_contact_data->m_otherIdentForm.ref(new epp_string(extraDataMap["otheridentform"]));
                }  

		l_req->m_cmd.ref(new epp_Command(NULL,
					asia_contact_extension,
					epp_trid(trID)));

		if ( extraDataMap.find("contact_id") == extraDataMap.end() ) {
			l_req->m_id.ref(new epp_string("sh1020"));
		}
		else {
			l_req->m_id.ref(new epp_string(extraDataMap["contact_id"]));
		}

		epp_ContactCreate_ref l_obj(new epp_ContactCreate());

		l_obj->setRequestData(*l_req);

		test.processAction(l_obj);

		epp_ContactCreateRsp_ref l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
		printRsp(l_rsp);
#endif
                // Update command   
                epp_ContactUpdateReq_ref l_req1(new epp_ContactUpdateReq());

                AsiaCEDContact_ref asia_extension(new AsiaCEDContact());
                asia_extension->m_command.ref(new epp_string("update"));
                asia_extension->m_ced_contact_data.ref(new AsiaCEDContactData());

                asia_extension->m_ced_contact_data->m_chg.ref(new epp_boolean(true));

                if ( extraDataMap.find("cclocality") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_ccLocality.ref(new epp_string("cn"));
                }
                else {
                     asia_extension->m_ced_contact_data->m_ccLocality.ref(new epp_string(extraDataMap["cclocality"]));
                }
                if ( extraDataMap.find("localitysp") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_localitySp.ref(new epp_string("ontario"));
                } else {
                     asia_extension->m_ced_contact_data->m_localitySp.ref(new epp_string(extraDataMap["localitysp"]));
                }
                if ( extraDataMap.find("localitycity") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_localityCity.ref(new epp_string("toronto"));
                } else {
                     asia_extension->m_ced_contact_data->m_localityCity.ref(new epp_string(extraDataMap["localitycity"]));
                }
                if ( extraDataMap.find("legalentitytype") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_legalEntityType.ref(new epp_string("naturalPerson"));
                } else {
                     asia_extension->m_ced_contact_data->m_legalEntityType.ref(new epp_string(extraDataMap["legalentitytype"]));
                }
                if ( extraDataMap.find("identform") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_identForm.ref(new epp_string("passport"));
                } else {
                     asia_extension->m_ced_contact_data->m_identForm.ref(new epp_string(extraDataMap["identform"]));
                }
                if ( extraDataMap.find("identnumber") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_identNumber.ref(new epp_string("12345678"));
                } else {
                     asia_extension->m_ced_contact_data->m_identNumber.ref(new epp_string(extraDataMap["identnumber"]));
                }
                if ( extraDataMap.find("otherletype") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_otherLEType.ref(new epp_string("naturalPerson"));
                } else {
                     asia_extension->m_ced_contact_data->m_otherLEType.ref(new epp_string(extraDataMap["otherletype"]));
                }
                if ( extraDataMap.find("otheridentform") == extraDataMap.end() ) {
                     asia_extension->m_ced_contact_data->m_otherIdentForm.ref(new epp_string("certificate"));                } else {
                     asia_extension->m_ced_contact_data->m_otherIdentForm.ref(new epp_string(extraDataMap["otheridentform"]));
                }
 
                l_req1->m_cmd.ref(new epp_Command(NULL,asia_extension,epp_trid("ABC-12345")));

                if ( extraDataMap.find("contact_id") == extraDataMap.end() ) {
                        l_req1->m_id.ref(new epp_string("sh1020"));
                }
                else {
                        l_req1->m_id.ref(new epp_string(extraDataMap["contact_id"]));
                }

                epp_ContactUpdate_ref l_obj1(new epp_ContactUpdate());

                l_obj1->setRequestData(*l_req1);

                test.processAction(l_obj1);

                epp_ContactUpdateRsp_ref l_rsp1 = l_obj1->getResponseData();

                printRsp(l_rsp1); 

                // Info command 
     	        epp_ContactInfoReq_ref contact_info_req(new epp_ContactInfoReq());

		contact_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
					NULL,
					eppobject::epp::epp_trid(trID)));

		if(extraDataMap.find("contact_id") == extraDataMap.end()) {
			contact_info_req->m_id.ref(new eppobject::epp::epp_string("sh1020"));
		}
		else {
			contact_info_req->m_id.ref(new eppobject::epp::epp_string(extraDataMap["contact_id"]));
		}

		epp_ContactInfo_ref contact_info_obj(new epp_ContactInfo());

		contact_info_obj->setRequestData(*contact_info_req);

		test.processAction(contact_info_obj);

		epp_ContactInfoRsp_ref contact_info_rsp = contact_info_obj->getResponseData();
		if(contact_info_rsp->m_rsp->m_ext_strings != NULL) {
			cout << "==== Contact Info Response Extenstion ===== \n";
			AsiaCEDContact_ref extension(new AsiaCEDContact());
			eppobject::epp::epp_xml_string_seq_ref extension_strings;
			extension_strings = contact_info_rsp->m_rsp->m_ext_strings;
			eppobject::epp::epp_xml_string first_extension = extension_strings->front();

			extension->fromXML(first_extension);
                        if (extension->m_ced_contact_data->m_ccLocality != NULL)
			  cout << "asia:ccLocality - \"" << *extension->m_ced_contact_data->m_ccLocality << "\"\n";
                        if (extension->m_ced_contact_data->m_localitySp != NULL)
                          cout << "asia:localitySp - \"" << *extension->m_ced_contact_data->m_localitySp << "\"\n";
                        if (extension->m_ced_contact_data->m_localityCity != NULL)
                          cout << "asia:localityCity - \"" << *extension->m_ced_contact_data->m_localityCity << "\"\n";
                        if (extension->m_ced_contact_data->m_legalEntityType != NULL)
                          cout << "asia:legalEntityType - \"" << *extension->m_ced_contact_data->m_legalEntityType << "\"\n";
                        if (extension->m_ced_contact_data->m_identForm != NULL)
                          cout << "asia:identForm - \"" << *extension->m_ced_contact_data->m_identForm << "\"\n";
                        if (extension->m_ced_contact_data->m_identNumber != NULL)
                          cout << "asia:identNumber - \"" << *extension->m_ced_contact_data->m_identNumber << "\"\n";
                        if (extension->m_ced_contact_data->m_otherLEType != NULL)
                          cout << "asia:otherLEType - \"" << *extension->m_ced_contact_data->m_otherLEType << "\"\n";
                        if (extension->m_ced_contact_data->m_otherIdentForm != NULL)
                          cout << "asia:otherIdentForm - \"" << *extension->m_ced_contact_data->m_otherIdentForm << "\"\n";
		}
		return 0;

	}
	catch(const epp_XMLException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(const epp_Exception & ex) {
		cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
		cerr << "<response>" << endl;
		printResultsSeq(ex.m_details);
		printTransID(ex.m_trans_id);
		cerr << "</response>" << endl;
#endif
	}
	catch(const epptransport::epp_TrException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(...) {
		cout << "Something else went wrong...\n";
	}
}
