/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/liberty-rtk-addon/c++/org/src/main_asia_iprexample.cc,v 1.3 2008/03/14 19:17:21 asimbirt Exp $ */

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_DomainCreateReq.h"
#include "epp-rtk-cpp/data/epp_DomainCreateRsp.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"
#include "epp-rtk-cpp/epp_DomainUpdate.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "epp-rtk-cpp/data/epp_DomainInfoReq.h"
#include "epp-rtk-cpp/data/epp_DomainInfoRsp.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "Ipr.h"
#include "AsiaDomain.h"

#include "epp-rtk-cpp/epp_ContactCheck.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCreateRsp_ref & req)
{
	cout << "Printing Response:\n_______________________________\n";

	if(req != NULL) {

		printResponsePart1(req->m_rsp);
		printResDataHeaders("domain", "creData");

		if(req->m_name != NULL)
			cerr << "<domain:name>" << *req-> m_name << "</domain:name>" << endl;

		if(req->m_creation_date != NULL)
			cerr << "<domain:crDate>" << *req-> m_creation_date << "</domain:crDate>" << endl;

		if(req->m_expiration_date != NULL)
			cerr << "<domain:exDate>" << *req-> m_expiration_date << "</domain:exDate>" << endl;

		printResDataFooter("domain", "creData");
		printResponsePart2(req->m_rsp);
	}


	cout << "_______________________________\n";
} // printRsp

int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;

        string domainRoid;

	string extra_usage = "[-extradata=trid=xxx:domain=abc.tld:period=1:ns1=asdf:ns2=asdf{:...}:reg=abc:contact1=t/abc{:...}:authinfo=pw/38239:iprType=SR2A:preVerified=0]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

	try {
		epp_DomainCreateReq_ref l_req(new epp_DomainCreateReq());

		string trID = "ABC-12345";
		if(extraDataMap.find("trid") != extraDataMap.end()) {
			trID = extraDataMap["trid"];
		}

		Ipr_ref ipr_extension(new Ipr());

		ipr_extension->m_command.ref(new epp_string("create"));
                ipr_extension->m_tld.ref(new epp_string("asia")); 
		ipr_extension->m_ipr_data.ref(new IprData());

                ipr_extension->m_ipr_data->m_name.ref(new epp_string("My Ipr"));
                ipr_extension->m_ipr_data->m_number.ref(new epp_string("12345566"));
                ipr_extension->m_ipr_data->m_country.ref(new epp_string("CA"));
                ipr_extension->m_ipr_data->m_reg_date.ref(new epp_string("2001-01-01"));
                ipr_extension->m_ipr_data->m_app_date.ref(new epp_string("2002-01-01"));
                ipr_extension->m_ipr_data->m_class.ref(new epp_string("1"));
                ipr_extension->m_ipr_data->m_entitlement.ref(new epp_string("owner"));
                ipr_extension->m_ipr_data->m_form.ref(new epp_string("corporation"));

                if ( extraDataMap.find("iprType") == extraDataMap.end() ) {
                        ipr_extension->m_ipr_data->m_type.ref(new epp_string("SR2A"));
                }
                else {
                        ipr_extension->m_ipr_data->m_type.ref(new epp_string(extraDataMap["iprType"]));
                }

                if ( extraDataMap.find("preVerified") == extraDataMap.end() ) {
                        ipr_extension->m_ipr_data->m_preVerified.ref(new epp_string("code"));
                }
                else {
                        ipr_extension->m_ipr_data->m_preVerified.ref(new epp_string(extraDataMap["preVerified"]));
                }

		l_req->m_cmd.ref(new epp_Command(NULL,
					ipr_extension,
					epp_trid(trID)));

		if ( extraDataMap.find("domain") == extraDataMap.end() ) {
			l_req->m_name.ref(new epp_string("example.asia"));
		}
		else {
			l_req->m_name.ref(new epp_string(extraDataMap["domain"]));
		}

		l_req->m_period.ref(new epp_DomainPeriod(YEAR, 2));

		// we don't care about name servers

		if ( extraDataMap.find("reg") == extraDataMap.end() ) {
			l_req->m_registrant.ref(new epp_string("ClientY001"));
		}
		else {
			l_req->m_registrant.ref(new epp_string(extraDataMap["reg"]));
		}

		l_req->m_contacts.ref(new epp_domain_contact_seq());

		string contact_id = "ClientY002";

		if ( extraDataMap.find("contact") != extraDataMap.end() ) {
			contact_id = extraDataMap["contact"];
		}

		epp_DomainContact Cont1(ADMIN, epp_string(contact_id));
		epp_DomainContact Cont2(TECH,  epp_string(contact_id));
		l_req->m_contacts->push_back(Cont1);
		l_req->m_contacts->push_back(Cont2);

		l_req->m_auth_info.ref(new epp_AuthInfo());
		l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
		l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

		epp_DomainCreate_ref l_obj(new epp_DomainCreate());

		l_obj->setRequestData(*l_req);

		test.processAction(l_obj);

		epp_DomainCreateRsp_ref l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
		printRsp(l_rsp);
#endif
                 if(l_rsp->m_rsp->m_ext_strings != NULL) {
                    AsiaDomain_ref asiad(new AsiaDomain());

                    eppobject::epp::epp_xml_string_seq_ref extension_strings;
                    extension_strings = l_rsp->m_rsp->m_ext_strings;
                    eppobject::epp::epp_xml_string first_extension = extension_strings->front();

                    asiad->fromXML(first_extension);

                    cout << "domainRoid: " << *asiad->m_domainRoid << endl;
                    domainRoid = *asiad->m_domainRoid;
                }
		
                // Info command 
                epp_DomainInfoReq_ref domain_info_req(new epp_DomainInfoReq());

                if (!domainRoid.empty()) {
                   AsiaDomain_ref asia_extension2(new AsiaDomain());
                   asia_extension2->m_command.ref(new epp_string("info"));

                   asia_extension2->m_domainRoid.ref(new epp_string(domainRoid));

		   domain_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
					asia_extension2,
					eppobject::epp::epp_trid(trID)));
 
                } else {
                   domain_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
                                        NULL,
                                        eppobject::epp::epp_trid(trID)));
                }

		if(extraDataMap.find("domain") == extraDataMap.end()) {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string("example.asia"));
		}
		else {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["domain"]));
		}

		domain_info_req->m_hosts_type.ref(new epp_DomainHostsType(eppobject::domain::ALL));

		epp_DomainInfo_ref domain_info_obj(new epp_DomainInfo());

		domain_info_obj->setRequestData(*domain_info_req);

		test.processAction(domain_info_obj);

		epp_DomainInfoRsp_ref domain_info_rsp = domain_info_obj->getResponseData();
		if(domain_info_rsp->m_rsp->m_ext_strings != NULL) {
			cout << "==== Domain Info Response Extension ===== \n";
			Ipr_ref extension(new Ipr);
			//eppobject::epp::epp_xml_string_seq_ref extension_strings;
			//extension_strings = domain_info_rsp->m_rsp->m_ext_strings;
			//eppobject::epp::epp_xml_string first_extension = extension_strings->front();

                        for(epp_xml_string_seq::iterator i = domain_info_rsp->m_rsp->m_ext_strings->begin();
                        i != domain_info_rsp->m_rsp->m_ext_strings->end(); i++)
                        {
                            DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+epp_string(*i)+"</parseme>");

                            DOM_Node parsemeNode = domtools::getSingleTag(doc, "parseme");
                            if(parsemeNode.isNull())
                            {
                                throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
                            }

                            DOM_Node iprNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("ipr:infData").item(0);

                            if (!iprNode.isNull()) {
			       extension->fromXML(epp_string(*i));
                               if (extension->m_ipr_data->m_name != NULL)
                                  cout << "ipr:name - \"" << *extension->m_ipr_data->m_name << "\"\n";
                               if (extension->m_ipr_data->m_number != NULL)
                                  cout << "ipr:number - \"" << *extension->m_ipr_data->m_number << "\"\n";
                               if (extension->m_ipr_data->m_country != NULL)
                                  cout << "ipr:ccLocality - \"" << *extension->m_ipr_data->m_country << "\"\n";
                               if (extension->m_ipr_data->m_app_date != NULL)
                                  cout << "ipr:appDate - \"" << *extension->m_ipr_data->m_app_date << "\"\n";
                               if (extension->m_ipr_data->m_reg_date != NULL)
                                  cout << "ipr:regDate - \"" << *extension->m_ipr_data->m_reg_date << "\"\n";
                               if (extension->m_ipr_data->m_class != NULL)
                                  cout << "ipr:class - \"" << *extension->m_ipr_data->m_class << "\"\n";
                               if (extension->m_ipr_data->m_entitlement != NULL)
                                  cout << "ipr:entitlement - \"" << *extension->m_ipr_data->m_entitlement << "\"\n";
                               if (extension->m_ipr_data->m_form != NULL)
                                  cout << "ipr:form - \"" << *extension->m_ipr_data->m_form << "\"\n";
                               if (extension->m_ipr_data->m_type != NULL)
                                  cout << "ipr:type - \"" << *extension->m_ipr_data->m_type << "\"\n";
                               if (extension->m_ipr_data->m_preVerified != NULL)
                                  cout << "ipr:preVerified - \"" << *extension->m_ipr_data->m_preVerified << "\"\n";
                            }
	                }	
             }
             
	     return 0;
	}
	catch(const epp_XMLException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(const epp_Exception & ex) {
		cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
		cerr << "<response>" << endl;
		printResultsSeq(ex.m_details);
		printTransID(ex.m_trans_id);
		cerr << "</response>" << endl;
#endif
	}
	catch(const epptransport::epp_TrException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(...) {
		cout << "Something else went wrong...\n";
	}
}
