/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/liberty-rtk-addon/c++/org/src/main_new_trademarkexample.cc,v 1.2 2006/07/05 14:19:26 ewang2004 Exp $ */

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_DomainCreateReq.h"
#include "epp-rtk-cpp/data/epp_DomainCreateRsp.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "epp-rtk-cpp/data/epp_DomainInfoReq.h"
#include "epp-rtk-cpp/data/epp_DomainInfoRsp.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "Trademark.h"

#include "epp-rtk-cpp/epp_ContactCheck.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCreateRsp_ref & req)
{
	cout << "Printing Response:\n_______________________________\n";

	if(req != NULL) {

		printResponsePart1(req->m_rsp);
		printResDataHeaders("domain", "creData");

		if(req->m_name != NULL)
			cerr << "<domain:name>" << *req-> m_name << "</domain:name>" << endl;

		if(req->m_creation_date != NULL)
			cerr << "<domain:crDate>" << *req-> m_creation_date << "</domain:crDate>" << endl;

		if(req->m_expiration_date != NULL)
			cerr << "<domain:exDate>" << *req-> m_expiration_date << "</domain:exDate>" << endl;

		printResDataFooter("domain", "creData");
		printResponsePart2(req->m_rsp);
	}


	cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;

	string extra_usage = "[-extradata=trid=xxx:domain=abc.tld:period=1:ns1=asdf:ns2=asdf{:...}:reg=abc:contact1=t/abc{:...}:authinfo=pw/38239]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

	try {

		epp_DomainCreateReq_ref l_req(new epp_DomainCreateReq());

		string trID = "ABC-12345";
		if(extraDataMap.find("trid") != extraDataMap.end()) {
			trID = extraDataMap["trid"];
		}

		Trademark_ref trademark_extension(new Trademark());

		trademark_extension->m_command.ref(new epp_string("create"));
		trademark_extension->m_tld.ref(new epp_string("mobi"));
		trademark_extension->m_trademark_data.ref(new TrademarkData());

        trademark_extension->m_trademark_data->m_name.ref(new epp_string("My Trademark"));
        trademark_extension->m_trademark_data->m_number.ref(new epp_string("12345566"));
        trademark_extension->m_trademark_data->m_country.ref(new epp_string("CA"));
        trademark_extension->m_trademark_data->m_reg_date.ref(new epp_string("2001-01-01"));
        trademark_extension->m_trademark_data->m_app_date.ref(new epp_string("2002-01-01"));

		l_req->m_cmd.ref(new epp_Command(NULL,
					trademark_extension,
					epp_trid(trID)));

		if ( extraDataMap.find("domain") == extraDataMap.end() ) {
			l_req->m_name.ref(new epp_string("example.mobi"));
		}
		else {
			l_req->m_name.ref(new epp_string(extraDataMap["domain"]));
		}

		l_req->m_period.ref(new epp_DomainPeriod(YEAR, 2));

		// we don't care about name servers

		if ( extraDataMap.find("reg") == extraDataMap.end() ) {
			l_req->m_registrant.ref(new epp_string("ClientY001"));
		}
		else {
			l_req->m_registrant.ref(new epp_string(extraDataMap["reg"]));
		}

		l_req->m_contacts.ref(new epp_domain_contact_seq());

		string contact_id = "ClientY002";

		if ( extraDataMap.find("contact") != extraDataMap.end() ) {
			contact_id = extraDataMap["contact"];
		}

		epp_DomainContact Cont1(ADMIN, epp_string(contact_id));
		epp_DomainContact Cont2(TECH,  epp_string(contact_id));
		l_req->m_contacts->push_back(Cont1);
		l_req->m_contacts->push_back(Cont2);

		l_req->m_auth_info.ref(new epp_AuthInfo());
		l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
		l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

		epp_DomainCreate_ref l_obj(new epp_DomainCreate());

		l_obj->setRequestData(*l_req);

		test.processAction(l_obj);

		epp_DomainCreateRsp_ref l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
		printRsp(l_rsp);
#endif
		epp_DomainInfoReq_ref domain_info_req(new epp_DomainInfoReq());

		domain_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
					NULL,
					eppobject::epp::epp_trid(trID)));

		if(extraDataMap.find("domain") == extraDataMap.end()) {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string("example.mobi"));
		}
		else {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["domain"]));
		}

		domain_info_req->m_hosts_type.ref(new epp_DomainHostsType(eppobject::domain::ALL));

		epp_DomainInfo_ref domain_info_obj(new epp_DomainInfo());

		domain_info_obj->setRequestData(*domain_info_req);

		test.processAction(domain_info_obj);

		epp_DomainInfoRsp_ref domain_info_rsp = domain_info_obj->getResponseData();
		if(domain_info_rsp->m_rsp->m_ext_strings != NULL) {
			cout << "==== Domain Info Response Extenstion ===== \n";
			Trademark_ref extension(new Trademark);
			eppobject::epp::epp_xml_string_seq_ref extension_strings;
			extension_strings = domain_info_rsp->m_rsp->m_ext_strings;
			eppobject::epp::epp_xml_string first_extension = extension_strings->front();

			extension->fromXML(first_extension);
            if (extension->m_trademark_data->m_name != NULL)
                cout << "trademark:name - \"" << *extension->m_trademark_data->m_name << "\"\n";
            if (extension->m_trademark_data->m_number != NULL)
                cout << "trademark:number - \"" << *extension->m_trademark_data->m_number << "\"\n";
            if (extension->m_trademark_data->m_country != NULL)
                cout << "trademark:country - \"" << *extension->m_trademark_data->m_country << "\"\n";
            if (extension->m_trademark_data->m_date != NULL)
                cout << "trademark:appDate - \"" << *extension->m_trademark_data->m_date << "\"\n";
            if (extension->m_trademark_data->m_app_date != NULL)
                cout << "trademark:appDate - \"" << *extension->m_trademark_data->m_app_date << "\"\n";
            if (extension->m_trademark_data->m_reg_date != NULL)
                cout << "trademark:regDate - \"" << *extension->m_trademark_data->m_reg_date << "\"\n";
		}
		return 0;

	}
	catch(const epp_XMLException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(const epp_Exception & ex) {
		cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
		cerr << "<response>" << endl;
		printResultsSeq(ex.m_details);
		printTransID(ex.m_trans_id);
		cerr << "</response>" << endl;
#endif
	}
	catch(const epptransport::epp_TrException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(...) {
		cout << "Something else went wrong...\n";
	}
}
