#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_DomainCreateReq.h"
#include "epp-rtk-cpp/data/epp_DomainCreateRsp.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "epp-rtk-cpp/data/epp_DomainInfoReq.h"
#include "epp-rtk-cpp/data/epp_DomainInfoRsp.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/data/epp_DomainUpdateReq.h"
#include "epp-rtk-cpp/data/epp_DomainUpdateRsp.h"
#include "epp-rtk-cpp/data/epp_eppdata.h"
#include "epp-rtk-cpp/epp_DomainUpdate.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "OxrsDomain.h"
#include "Ipr.h"
#include "IprData.h"

#include "epp-rtk-cpp/epp_ContactCheck.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCreateRsp_ref & req)
{
	cout << "Printing Response:\n_______________________________\n";

	if(req != NULL) {

		printResponsePart1(req->m_rsp);
		printResDataHeaders("domain", "creData");

		if(req->m_name != NULL)
			cerr << "<domain:name>" << *req-> m_name << "</domain:name>" << endl;

		if(req->m_creation_date != NULL)
			cerr << "<domain:crDate>" << *req-> m_creation_date << "</domain:crDate>" << endl;

		if(req->m_expiration_date != NULL)
			cerr << "<domain:exDate>" << *req-> m_expiration_date << "</domain:exDate>" << endl;

		printResDataFooter("domain", "creData");
		printResponsePart2(req->m_rsp);

	}


	cout << "_______________________________\n";
} // printRsp

void printRsp(const epp_DomainUpdateRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";

   if(req != NULL) {

      printResponsePart(req->m_rsp);
   }


   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;
        string domainRoid;

	string extra_usage = "[-extradata=trid=xxx:domain=abc.tld:period=1:ns1=asdf:ns2=asdf{:...}:reg=abc:contact1=t/abc{:...}:authinfo=pw/38239:maintainerUrl=www.afilias.info:iprType=SUNRISE]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

	try {

		epp_DomainCreateReq_ref l_req(new epp_DomainCreateReq());

		string trID = "ABC-12345";
		if(extraDataMap.find("trid") != extraDataMap.end()) {
			trID = extraDataMap["trid"];
		}

		OxrsDomain_ref oxrs_domain_extension(new OxrsDomain());
		oxrs_domain_extension->m_command.ref(new epp_string("create"));

                if ( extraDataMap.find("maintainerUrl") == extraDataMap.end() ) {
                        oxrs_domain_extension->m_maintainerUrl.ref(new epp_string("www.afilias.info"));
                }
                else {
                        oxrs_domain_extension->m_maintainerUrl.ref(new epp_string(extraDataMap["maintainerUrl"]));
                }

                Ipr_ref ipr_extension(new Ipr()); 
                ipr_extension->m_command.ref(new epp_string("create"));
                ipr_extension->m_tld.ref(new epp_string("oxrs"));
                ipr_extension->m_ipr_data.ref(new IprData());

                ipr_extension->m_ipr_data->m_name.ref(new epp_string("My Ipr"));
                ipr_extension->m_ipr_data->m_number.ref(new epp_string("12345566"));
                ipr_extension->m_ipr_data->m_country.ref(new epp_string("CA"));
                ipr_extension->m_ipr_data->m_reg_date.ref(new epp_string("2001-01-01"));
                ipr_extension->m_ipr_data->m_app_date.ref(new epp_string("2002-01-01"));
 
                if ( extraDataMap.find("iprType") == extraDataMap.end() ) {
                        ipr_extension->m_ipr_data->m_type.ref(new epp_string("SUNRISE"));
                }
                else {
                        ipr_extension->m_ipr_data->m_type.ref(new epp_string(extraDataMap["iprType"]));
                }

                l_req->m_cmd.ref(new epp_Command(NULL,NULL,epp_trid(trID)));

                l_req->m_cmd->m_extentions.ref(new epp_extension_ref_seq());
      
                l_req->m_cmd->m_extentions->push_back(oxrs_domain_extension); 
                l_req->m_cmd->m_extentions->push_back(ipr_extension);
 
		if ( extraDataMap.find("domain") == extraDataMap.end() ) {
			l_req->m_name.ref(new epp_string("example.me"));
		}
		else {
			l_req->m_name.ref(new epp_string(extraDataMap["domain"]));
		}

		l_req->m_period.ref(new epp_DomainPeriod(YEAR, 2));

		// we don't care about name servers

		if ( extraDataMap.find("reg") == extraDataMap.end() ) {
			l_req->m_registrant.ref(new epp_string("ClientY001"));
		}
		else {
			l_req->m_registrant.ref(new epp_string(extraDataMap["reg"]));
		}

		l_req->m_contacts.ref(new epp_domain_contact_seq());

		string contact_id = "ClientY002";

		if ( extraDataMap.find("contact") != extraDataMap.end() ) {
			contact_id = extraDataMap["contact"];
		}

		epp_DomainContact Cont1(ADMIN, epp_string(contact_id));
		epp_DomainContact Cont2(TECH,  epp_string(contact_id));
		l_req->m_contacts->push_back(Cont1);
		l_req->m_contacts->push_back(Cont2);

		l_req->m_auth_info.ref(new epp_AuthInfo());
		l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
		l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));

		epp_DomainCreate_ref l_obj(new epp_DomainCreate());

		l_obj->setRequestData(*l_req);

		test.processAction(l_obj);

		epp_DomainCreateRsp_ref l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
		printRsp(l_rsp);
#endif
	        if(l_rsp->m_rsp->m_ext_strings != NULL) {
                    OxrsDomain_ref oxrsd(new OxrsDomain());

                    eppobject::epp::epp_xml_string_seq_ref extension_strings;
                    extension_strings = l_rsp->m_rsp->m_ext_strings;
                    eppobject::epp::epp_xml_string first_extension = extension_strings->front();

                    oxrsd->fromXML(first_extension);

                    cout << "domainRoid: " << *oxrsd->m_domainRoid << endl;
                    domainRoid = *oxrsd->m_domainRoid;  
                }
 
                // Update command
                epp_DomainUpdateReq_ref l_req1(new epp_DomainUpdateReq());

                OxrsDomain_ref oxrs_extension(new OxrsDomain());

                oxrs_extension->m_command.ref(new epp_string("update"));
                
                oxrs_extension->m_chg.ref(new epp_boolean(true));

                oxrs_extension->m_domainRoid.ref(new epp_string(domainRoid));  

                if ( extraDataMap.find("maintainerUrl") == extraDataMap.end() ) {
                        oxrs_extension->m_maintainerUrl.ref(new epp_string("www.afilias.info"));
                }
                else {
                        oxrs_extension->m_maintainerUrl.ref(new epp_string(extraDataMap["maintainerUrl"]));
                }

                l_req1->m_cmd.ref(new epp_Command(NULL,oxrs_extension,epp_trid("ABC-12345")));

                if ( extraDataMap.find("domain") == extraDataMap.end() ) {
                        l_req1->m_name.ref(new epp_string("example.me"));
                }
                else {
                        l_req1->m_name.ref(new epp_string(extraDataMap["domain"]));
                }

                epp_DomainUpdate_ref l_obj1(new epp_DomainUpdate());

                l_obj1->setRequestData(*l_req1);

                test.processAction(l_obj1);

                epp_DomainUpdateRsp_ref l_rsp1 = l_obj1->getResponseData();

                printRsp(l_rsp1);

                // Info command 
     	        epp_DomainInfoReq_ref domain_info_req(new epp_DomainInfoReq());

                OxrsDomain_ref oxrs_extension2(new OxrsDomain());
                oxrs_extension2->m_command.ref(new epp_string("info"));

                oxrs_extension2->m_domainRoid.ref(new epp_string(domainRoid));

		domain_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
					oxrs_extension2,
					eppobject::epp::epp_trid(trID)));

		if(extraDataMap.find("domain") == extraDataMap.end()) {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string("example.me"));
		}
		else {
			domain_info_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["domain"]));
		}

		domain_info_req->m_hosts_type.ref(new epp_DomainHostsType(eppobject::domain::ALL));

		epp_DomainInfo_ref domain_info_obj(new epp_DomainInfo());

		domain_info_obj->setRequestData(*domain_info_req);

		test.processAction(domain_info_obj);

		epp_DomainInfoRsp_ref domain_info_rsp = domain_info_obj->getResponseData();
		if(domain_info_rsp->m_rsp->m_ext_strings != NULL) {
			cout << "==== Domain Info Response Extension ===== \n";
			OxrsDomain_ref extension(new OxrsDomain());
			//eppobject::epp::epp_xml_string_seq_ref extension_strings;
			//extension_strings = domain_info_rsp->m_rsp->m_ext_strings;
			//eppobject::epp::epp_xml_string first_extension = extension_strings->front();

                        for(epp_xml_string_seq::iterator i = domain_info_rsp->m_rsp->m_ext_strings->begin();
                        i != domain_info_rsp->m_rsp->m_ext_strings->end(); i++)
                        {
                            DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+epp_string(*i)+"</parseme>");

                            DOM_Node parsemeNode = domtools::getSingleTag(doc, "parseme");
                            if(parsemeNode.isNull())
                            {
                                throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
                            }

                            DOM_Node oxrsNode1 = ((DOM_Element*)&parsemeNode)->getElementsByTagName("oxrs:infData").item(0);

                            if (!oxrsNode1.isNull()) {
                               extension->fromXML(epp_string(*i));

                               if (extension->m_maintainerUrl != NULL)  
                                  cout << "oxrs:maintainerUrl - \"" << *extension->m_maintainerUrl << "\"\n";
                           }
                      }
		}
		return 0;

	}
	catch(const epp_XMLException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(const epp_Exception & ex) {
		cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
		cerr << "<response>" << endl;
		printResultsSeq(ex.m_details);
		printTransID(ex.m_trans_id);
		cerr << "</response>" << endl;
#endif
	}
	catch(const epptransport::epp_TrException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(...) {
		cout << "Something else went wrong...\n";
	}
}
