/************************************************************************
 * EPP RTK C++
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact information:  epprtk@gnr.com
 *
 *                       EPP RTK
 *                       The Global Name Registry, Limited
 *                       125 High Holborn
 *                       London WC1V 6QA
 *                       United Kingdom
 ************************************************************************/

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_DomainCreateReq.h"
#include "epp-rtk-cpp/data/epp_DomainCreateRsp.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainUpdate.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "epp-rtk-cpp/data/epp_DomainInfoReq.h"
#include "epp-rtk-cpp/data/epp_DomainInfoRsp.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "Rgp.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainUpdateRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";

   if(req != NULL) {

      printResponsePart(req->m_rsp);
   }


   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {

	epp_Session test;

	string username, password;

	string extra_usage = "[-extradata=trid=xxx:domain=abc.tld]";
	if(!handleArgs(test, argc, argv, username, password, extra_usage))
		return 1;

	map<string,string> extraDataMap;

	if(argc > 1) {
		for(int parm = 1; parm < argc; parm++) {

			if (!strncmp(argv[parm], "-extradata=", 11)) {
#ifdef RTK_DEBUG
				cout << "Found extra data param..." << endl;
#endif
				string td = &argv[parm][11];
				extraDataMap = getExtraDataMap(td);
			}
		}
	}

	testLogIn(test, username, password);

	try {
		string trID = "ABC-12345";
		if(extraDataMap.find("trid") != extraDataMap.end()) {
			trID = extraDataMap["trid"];
		}

                //Domain update
                epp_DomainUpdateReq_ref l_req(new epp_DomainUpdateReq());
                l_req->m_change.ref(new epp_DomainUpdateChange());

                Rgp_ref rgp_extension(new Rgp());
                rgp_extension->m_command.ref(new epp_string("update"));
                rgp_extension->m_restore_op.ref(new epp_string("request"));

                l_req->m_cmd.ref(new epp_Command(NULL,rgp_extension,epp_trid("ABC-12345")));

                if ( extraDataMap.find("domain") == extraDataMap.end() ) {
                        l_req->m_name.ref(new epp_string("example.org"));
                }
                else {
                        l_req->m_name.ref(new epp_string(extraDataMap["domain"]));
                }

                epp_DomainUpdate_ref l_obj(new epp_DomainUpdate());

                l_obj->setRequestData(*l_req);

                test.processAction(l_obj);

                epp_DomainUpdateRsp_ref l_rsp = l_obj->getResponseData();

                cout << "==== Domain Update Response (Request) ===== \n";

                printRsp(l_rsp);

                //Print response
                if(l_rsp->m_rsp->m_ext_strings != NULL) {
                    Rgp_ref rgp_r(new Rgp());

                    eppobject::epp::epp_xml_string_seq_ref extension_strings;
                    extension_strings = l_rsp->m_rsp->m_ext_strings;
                    eppobject::epp::epp_xml_string first_extension = extension_strings->front();

                    rgp_r->fromXML(first_extension);

                    unsigned int ii; 
                    cout << "Rgp statuses: " << endl;
                    for (ii = 0;ii < rgp_r->m_rgp_statuses.size(); ii++) {
                        cout <<  (rgp_r->m_rgp_statuses)[ii] << endl;
                    }
                }

                //Update with the report
                epp_DomainUpdateReq_ref l_req1(new epp_DomainUpdateReq());
                l_req1->m_change.ref(new epp_DomainUpdateChange());          
 
                Rgp_ref rgp_extension1(new Rgp());

                rgp_extension1->m_command.ref(new epp_string("update"));
                rgp_extension1->m_restore_op.ref(new epp_string("report"));
                rgp_extension1->m_rgp_report_data.ref(new RgpReportData());

                rgp_extension1->m_rgp_report_data->m_pre_data.ref(new epp_string("Pre-delete registration data goes here.Both XML and free text are allowed."));
                rgp_extension1->m_rgp_report_data->m_post_data.ref(new epp_string("Post-restore registration data goes here.Both XML and free text are allowed."));
 
                cout << "Please enter delete date and time (ex.2003-07-10T22:00:00.0Z): ";
                string delete_time;
                cin >> delete_time;
                rgp_extension1->m_rgp_report_data->m_del_time.ref(new epp_string(delete_time));

                cout << "Please enter restore date and time (ex.2003-07-10T22:00:00.0Z): ";
                string restore_time;
                cin >> restore_time;

                rgp_extension1->m_rgp_report_data->m_res_time.ref(new epp_string(restore_time)); 
                rgp_extension1->m_rgp_report_data->m_res_reason.ref(new epp_string("Registrant error."));
                rgp_extension1->m_rgp_report_data->m_statement_1.ref(new epp_string("This registrar has not restored the Registered Name in order to assume the rights to use or sell the Registered Name for itself or for any third party."));
                rgp_extension1->m_rgp_report_data->m_statement_2.ref(new epp_string("The information in this report is true to best of this registrar's knowledge, and this registrar acknowledges that intentionally supplying false information in this report shall constitute an incurable material breach of the Registry-Registrar Agreement."));
                rgp_extension1->m_rgp_report_data->m_other.ref(new epp_string("Supporting information goes here."));

                l_req1->m_cmd.ref(new epp_Command(NULL,rgp_extension1,epp_trid("ABC-12345")));

                if ( extraDataMap.find("domain") == extraDataMap.end() ) {
                        l_req1->m_name.ref(new epp_string("example.org"));
                }
                else {
                        l_req1->m_name.ref(new epp_string(extraDataMap["domain"]));
                }

                epp_DomainUpdate_ref l_obj1(new epp_DomainUpdate());

                l_obj1->setRequestData(*l_req1);

                test.processAction(l_obj1);

                epp_DomainUpdateRsp_ref l_rsp1 = l_obj1->getResponseData();

                cout << "==== Domain Update Response (Report) ===== \n";

                printRsp(l_rsp1);

                // Info command
                epp_DomainInfoReq_ref domain_info_req(new epp_DomainInfoReq());

                domain_info_req->m_cmd.ref(new eppobject::epp::epp_Command(NULL,
                                        NULL,
                                        eppobject::epp::epp_trid(trID)));

                if(extraDataMap.find("domain") == extraDataMap.end()) {
                        domain_info_req->m_name.ref(new eppobject::epp::epp_string("example.org"));
                }
                else {
                        domain_info_req->m_name.ref(new eppobject::epp::epp_string(extraDataMap["domain"]));
                }

                domain_info_req->m_hosts_type.ref(new epp_DomainHostsType(eppobject::domain::ALL));

                epp_DomainInfo_ref domain_info_obj(new epp_DomainInfo());

                domain_info_obj->setRequestData(*domain_info_req);

                test.processAction(domain_info_obj);

                epp_DomainInfoRsp_ref domain_info_rsp = domain_info_obj->getResponseData();
                if(domain_info_rsp->m_rsp->m_ext_strings != NULL) {
                        cout << "==== Domain Info Response Extension ===== \n";
                        Rgp_ref extension(new Rgp);
                        //eppobject::epp::epp_xml_string_seq_ref extension_strings;
                        //extension_strings = domain_info_rsp->m_rsp->m_ext_strings;
                        //eppobject::epp::epp_xml_string first_extension = extension_strings->front();

                        for(epp_xml_string_seq::iterator i = domain_info_rsp->m_rsp->m_ext_strings->begin();
                        i != domain_info_rsp->m_rsp->m_ext_strings->end(); i++)
                        {
                            DOM_Document doc = createDOM_Document("<parseme xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">"+epp_string(*i)+"</parseme>");

                            DOM_Node parsemeNode = domtools::getSingleTag(doc, "parseme");
                            if(parsemeNode.isNull())
                            {
                                throw epp_XMLException(__FILE__, __LINE__, "Unable to get parseme node");
                            }

                            DOM_Node rgpNode = ((DOM_Element*)&parsemeNode)->getElementsByTagName("rgp:infData").item(0);

                            if (!rgpNode.isNull()) {
                               extension->fromXML(epp_string(*i));

                               unsigned int ii;
                               cout << "Rgp statuses: " << endl;
                               for (ii = 0;ii < extension->m_rgp_statuses.size(); ii++) {
                                  cout <<  (extension->m_rgp_statuses)[ii] << endl;
                               }
                            }
                        }
                } 

		return 0;
	}
	catch(const epp_XMLException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(const epp_Exception & ex) {
		cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
		cerr << "<response>" << endl;
		printResultsSeq(ex.m_details);
		printTransID(ex.m_trans_id);
		cerr << "</response>" << endl;
#endif
	}
	catch(const epptransport::epp_TrException & ex) {
		cout << "Exception: " << ex.getString() << endl;
	}
	catch(...) {
		cout << "Something else went wrong...\n";
	}
}
