/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry 
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"
#include "epp-rtk-cpp/epp_DomainUpdate.h"
#include "epp-rtk-cpp/epp_DomainDelete.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"

#include "SecDNSCreate.h"
#include "SecDNSDsData.h"
#include "SecDNSKeyData.h"
#include "SecDNSUpdate.h"
#include "SecDNSUpdateAdd.h"
#include "SecDNSUpdateRem.h"
#include "SecDNSUpdateChg.h"
#include "SecDNSInfData.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCreateRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart(req->m_rsp);
      
      if(req->m_name != NULL)
    cout << "m_name: " << *req-> m_name << endl;
      if(req->m_expiration_date != NULL)
    cout << "m_expiration_date: " << *req-> m_expiration_date << endl;
   }
   
   
   cout << "_______________________________\n";
} // printRsp

void printRsp(const epp_DomainUpdateRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart(req->m_rsp);
   }
   
   
   cout << "_______________________________\n";
} // printRsp


void printRsp(const epp_DomainInfoRsp_ref req)
{
    cout << "Printing Response:\n_______________________________\n";
   
    if (req != NULL) 
    {
        printResponsePart(req->m_rsp);
      
        if (req->m_name != NULL) cout << "m_name: " << *req->m_name << endl;
      
        if (req->m_status != NULL)
        {
            cout << "m_status:\n";

            for (epp_domain_status_seq::iterator i = req->m_status->begin(); i != req->m_status->end(); i++)
            {
                if (i->m_type != NULL) 
                {
                    string status;
                    
                    switch (*i->m_type) 
                    {
                    case SERVER_HOLD: status = "SERVER_HOLD"; break;
                    case INACTIVE: status = "INACTIVE"; break;
                    case SERVER_DELETE_PROHIBITED: status = "SERVER_DELETE_PROHIBITED"; break;
                    case SERVER_RENEW_PROHIBITED: status = "SERVER_RENEW_PROHIBITED"; break;
                    case SERVER_TRANSFER_PROHIBITED: status = "SERVER_TRANSFER_PROHIBITED"; break;
                    case SERVER_UPDATE_PROHIBITED: status = "SERVER_UPDATE_PROHIBITED"; break;
                    case OK: status = "OK"; break;
                    case PENDING_DELETE: status = "PENDING_DELETE"; break;
                    case PENDING_TRANSFER: status = "PENDING_TRANSFER"; break;
                    case PENDING_CREATE: status = "PENDING_CREATE"; break;
                    case PENDING_RENEW: status = "PENDING_RENEW"; break;
                    case PENDING_UPDATE: status = "PENDING_UPDATE"; break;
                    case CLIENT_HOLD: status = "CLIENT_HOLD"; break;
                    case CLIENT_DELETE_PROHIBITED: status = "CLIENT_DELETE_PROHIBITED"; break;
                    case CLIENT_UPDATE_PROHIBITED: status = "CLIENT_UPDATE_PROHIBITED"; break;
                    case CLIENT_RENEW_PROHIBITED: status = "CLIENT_RENEW_PROHIBITED"; break;
                    case CLIENT_TRANSFER_PROHIBITED: status = "CLIENT_TRANSFER_PROHIBITED"; break;
                    default: status = "**UNKNOWN**";
                    }

                    cout << " m_type: " << status << endl;
                }
                
                if (i->m_lang != NULL) cout << " m_lang :" << *i->m_lang << endl;
                if (i->m_value != NULL) cout << " m_value: " << *i->m_value << endl;
            }
        }
      
        if (req->m_registrant != NULL) cout << "m_registrant: " << *req->m_registrant << endl;
      
        if (req->m_contacts != NULL)
        {
            cout << "m_contacts:\n";

            for (epp_domain_contact_seq::iterator i = req->m_contacts->begin(); i != req->m_contacts->end(); i++)
            {
                if (i->m_type != NULL) 
                {
                    string type;
                    
                    switch(*i->m_type) 
                    {
                    case ADMIN: type = "ADMIN"; break;
                    case BILLING: type = "BILLING"; break;
                    case TECH: type = "TECH"; break;
                    default: type = "**UNKNOWN**";
                    }
                    
                    cout << " m_type: " << type << endl;
                }
            }
        }
      
        if (req->m_name_servers != NULL)
        {
            cout << "m_name_servers:\n";
            
            for (epp_string_seq::iterator i = req->m_name_servers->begin(); i != req->m_name_servers->end(); i++)
            {
                cout << " ns: " << *i << endl;
            }
        }
      
        if (req->m_hosts != NULL)
        {
            cout << "m_hosts:\n";
            
            for (epp_string_seq::iterator i = req->m_hosts->begin(); i != req->m_hosts->end(); i++)
            {
                cout << " host: " << *i << endl;
            }
        }
      
        if (req->m_client_id != NULL) cout << "m_client_id: " << *req->m_client_id << endl;
        if (req->m_created_by != NULL) cout << "m_created_by: " << *req->m_created_by << endl;
        if (req->m_created_date != NULL) cout << "m_created_date: " << *req->m_created_date << endl;
        if (req->m_updated_by != NULL) cout << "m_updated_by: " << *req->m_updated_by << endl;
        if (req->m_updated_date != NULL) cout << "m_updated_date: " << *req->m_updated_date << endl;
        if (req->m_expiration_date != NULL) cout << "m_expiration_date: " << *req-> m_expiration_date << endl;
        if (req->m_transfer_date != NULL) cout << "m_transfer_date: " << *req->m_transfer_date << endl;
          
        if (req->m_auth_info != NULL) 
        {
            cout << "m_auth_info: " << endl;
         
            if (req->m_auth_info->m_type != NULL)
            {
                string type;
                
                switch(*req->m_auth_info->m_type) 
                {
                case PW: type = "PW"; break;
                default: type = "**UNKNOWN**";
                }

                cout << " m_type: " << type << endl;
            }

            if(req->m_auth_info->m_value != NULL) cout << " m_value: " << *req->m_auth_info->m_value << endl;
        }
    } // if(req != NULL)
   
    cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) 
{
    epp_Session test;
   
    string username, password;
   
    if (!handleArgs(test, argc, argv, username, password)) return 1;
   
    testLogIn(test, username, password);

    try 
    {
        cout << "Delete Command" << endl;
 
        epp_DomainDeleteReq_ref l_req(new epp_DomainDeleteReq());

        l_req->m_cmd.ref(new epp_Command(NULL, NULL, epp_trid("ABC-12346"))); 
        l_req->m_name.ref(new epp_string("acme.org"));
      
        epp_DomainDelete_ref l_obj(new epp_DomainDelete());
      
        l_obj->setRequestData(*l_req);
      
        test.processAction(l_obj);
      
        epp_DomainDeleteRsp_ref l_rsp = l_obj->getResponseData();
    }
    catch(epp_XMLException ex) 
    {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) 
    {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) 
    {
        cout << "Something else went wrong...\n";
    }

    try 
    {
        cout << "Create Command" << endl;

        epp_DomainCreateReq_ref l_req(new epp_DomainCreateReq());

        SecDNSDsData_ref ds_data(new SecDNSDsData());
        ds_data->m_ds_key_tag.ref(new epp_long(12345));
        ds_data->m_ds_alg.ref(new epp_short(3));
        ds_data->m_ds_digest_type.ref(new epp_short(1));
        ds_data->m_ds_digest.ref(new epp_string("33453453434553434534"));
        ds_data->m_ds_max_sig_life.ref(new epp_long(604800));
        ds_data->m_ds_key_data.ref(new SecDNSKeyData());
        ds_data->m_ds_key_data->m_ds_flags.ref(new epp_short(256));
        ds_data->m_ds_key_data->m_ds_protocol.ref(new epp_short(3));
        ds_data->m_ds_key_data->m_ds_alg.ref(new epp_short(1));
        ds_data->m_ds_key_data->m_ds_pub_key.ref(new epp_string("AQOyKAL4mM81OhKl9dlhJ8nw5m2S0z0EEpVtHo4AUzjxgU6mupRZXjjSnavw4QxCTFPuwTpGGqAG4WQCwKh5Sr1OJ/xAxUIdp5Sbi/P/ugUcdau1QJydq3yvxLIOTOv9FZuLRbpNPZGqXSfMD5kWTbZwtFS8jbmxJojoGp1xBSej0Q=="));

        SecDNSCreate_ref secdns_create(new SecDNSCreate());
        secdns_create->m_ds_data.ref(new SecDNSDsData_seq());
        secdns_create->m_ds_data->push_back(*ds_data);
            
        l_req->m_cmd.ref(new epp_Command(NULL,secdns_create,epp_trid("ABC-12345")));

        l_req->m_name.ref(new epp_string("acme.org"));
        l_req->m_period.ref(new epp_DomainPeriod(YEAR, 2));
      
        l_req->m_auth_info.ref(new epp_AuthInfo());
        l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
        l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));
      
        epp_DomainCreate_ref l_obj(new epp_DomainCreate());
      
        l_obj->setRequestData(*l_req);
      
        test.processAction(l_obj);
      
        epp_DomainCreateRsp_ref l_rsp = l_obj->getResponseData();
      
        printRsp(l_rsp);
    }
    catch(epp_XMLException ex) 
    {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) 
    {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) 
    {
        cout << "Something else went wrong...\n";
    }
   
    try 
    {
        cout << "Info Command" << endl;

        epp_DomainInfoReq_ref l_req(new epp_DomainInfoReq());

        l_req->m_cmd.ref(new epp_Command(NULL, NULL, epp_trid("ABC-12346"))); 
        l_req->m_name.ref(new epp_string("acme.org"));
      
        epp_DomainInfo_ref l_obj(new epp_DomainInfo());
      
        l_obj->setRequestData(*l_req);
      
        test.processAction(l_obj);
      
        epp_DomainInfoRsp_ref l_rsp = l_obj->getResponseData();
      
        printRsp(l_rsp);

        // Let us do something more with the unspec-object:
        if(l_rsp->m_rsp->m_ext_strings != NULL)
        {
            eppobject::epp::epp_xml_string_seq_ref extension_strings;
            extension_strings = l_rsp->m_rsp->m_ext_strings;
            eppobject::epp::epp_xml_string first_extension = extension_strings->front();
           
            SecDNSInfData_ref inf_data(new SecDNSInfData());
            inf_data->fromXML(first_extension);

            if (inf_data->m_ds_data != NULL) {
                 for(SecDNSDsData_seq::iterator j = inf_data->m_ds_data->begin();
                       j != inf_data->m_ds_data->end(); j++)
                 {
                     cerr << "<secDNS:dsData>" << endl;
                 
                     if (j->m_ds_key_tag != NULL) 
                       cerr << "<secDNS:keyTag>" << *j->m_ds_key_tag << "</secDNS:keyTag>" << endl;

                     if (j->m_ds_alg != NULL)
                       cerr << "<secDNS:alg>" << *j->m_ds_alg << "</secDNS:alg>" << endl;

                     if (j->m_ds_digest_type != NULL)
                       cerr << "<secDNS:digestType>"  << *j->m_ds_digest_type << "</secDNS:digestType>" << endl;
                     
                     if (j->m_ds_digest != NULL)
                       cerr << "<secDNS:digest>"  << *j->m_ds_digest << "</secDNS:digest>" << endl;

                     if (j->m_ds_max_sig_life != NULL)
                       cerr << "<secDNS:maxSigLife>"  << *j->m_ds_max_sig_life << "</secDNS:maxSigLife>" << endl;

                     if (j->m_ds_key_data != NULL) {                     
                       cerr << "<secDNS:keyData>" << endl;

                       if (j->m_ds_key_data->m_ds_flags != NULL) 
                          cerr << "<secDNS:flags>"  << *j->m_ds_key_data->m_ds_flags << "</secDNS:flags>" << endl;

                       if (j->m_ds_key_data->m_ds_protocol != NULL)
                          cerr << "<secDNS:protocol>"  << *j->m_ds_key_data->m_ds_protocol << "</secDNS:protocol>" << endl;

                       if (j->m_ds_key_data->m_ds_alg != NULL)
                          cerr << "<secDNS:alg>"  << *j->m_ds_key_data->m_ds_alg << "</secDNS:alg>" << endl;

                       if (j->m_ds_key_data->m_ds_pub_key != NULL)
                          cerr << "<secDNS:pubKey>"  << *j->m_ds_key_data->m_ds_pub_key << "</secDNS:pubKey>" << endl;

                       cerr << "</secDNS:keyData>" << endl;  

                     }


                     cerr << "</secDNS:dsData>" << endl;
                 }
            }
        }
    }
    catch(epp_XMLException ex) 
    {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) 
    {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) 
    {
        cout << "Something else went wrong...\n";
    }

    try 
    {
        epp_DomainUpdateReq_ref l_req(new epp_DomainUpdateReq());

        SecDNSDsData_ref ds_data1(new SecDNSDsData());
        ds_data1->m_ds_key_tag.ref(new epp_long(1234));
        ds_data1->m_ds_alg.ref(new epp_short(3));
        ds_data1->m_ds_digest_type.ref(new epp_short(1));
        ds_data1->m_ds_digest.ref(new epp_string("33453453434553434534"));
        ds_data1->m_ds_max_sig_life.ref(new epp_long(604800));
        ds_data1->m_ds_key_data.ref(new SecDNSKeyData());
        ds_data1->m_ds_key_data->m_ds_flags.ref(new epp_short(256));
        ds_data1->m_ds_key_data->m_ds_protocol.ref(new epp_short(3));
        ds_data1->m_ds_key_data->m_ds_alg.ref(new epp_short(1));
        ds_data1->m_ds_key_data->m_ds_pub_key.ref(new epp_string("AQOyKAL4mM81OhKl9dlhJ8nw5m2S0z0EEpVtHo4AUzjxgU6mupRZXjjSnavw4QxCTFPuwTpGGqAG4WQCwKh5Sr1OJ/xAxUIdp5Sbi/P/ugUcdau1QJydq3yvxLIOTOv9FZuLRbpNPZGqXSfMD5kWTbZwtFS8jbmxJojoGp1xBSej0Q=="));

        SecDNSDsData_ref ds_data2(new SecDNSDsData());
        ds_data2->m_ds_key_tag.ref(new epp_long(55555));
        ds_data2->m_ds_alg.ref(new epp_short(3));
        ds_data2->m_ds_digest_type.ref(new epp_short(1));
        ds_data2->m_ds_digest.ref(new epp_string("55555555555555555555"));
        ds_data2->m_ds_max_sig_life.ref(new epp_long(704850));
        ds_data2->m_ds_key_data.ref(new SecDNSKeyData());
        ds_data2->m_ds_key_data->m_ds_flags.ref(new epp_short(257));
        ds_data2->m_ds_key_data->m_ds_protocol.ref(new epp_short(3));
        ds_data2->m_ds_key_data->m_ds_alg.ref(new epp_short(1));
        ds_data2->m_ds_key_data->m_ds_pub_key.ref(new epp_string("AQOyKAL4mM81OhKl9dlhJ8nw5m2S0z0EEpVtHo4AUzjxgU6mupRZXjjSnavw4QxCTFPuwTpGGqAG4WQCwKh5Sr1OJ/xAxUIdp5Sbi/P/ugUcdau1QJydq3yvxLIOTOv9FZuLRbpNPZGqXSfMD5kWTbZwtFS8jbmxJojoGp1xBSej0Q=="));

        SecDNSDsData_ref ds_data3(new SecDNSDsData());
        ds_data3->m_ds_key_tag.ref(new epp_long(33333));
        ds_data3->m_ds_alg.ref(new epp_short(3));
        ds_data3->m_ds_digest_type.ref(new epp_short(1));
        ds_data3->m_ds_digest.ref(new epp_string("99999999999999999999"));

        epp_long kt1(1234);

        cout << "Add command" << endl; 

        SecDNSUpdate_ref secdns_update(new SecDNSUpdate());
        secdns_update->m_urgent = true;

        secdns_update->m_add.ref(new SecDNSUpdateAdd());
        secdns_update->m_add->m_ds_data.ref(new SecDNSDsData_seq());
        secdns_update->m_add->m_ds_data->push_back(*ds_data1);
        secdns_update->m_add->m_ds_data->push_back(*ds_data2);
        secdns_update->m_add->m_ds_data->push_back(*ds_data3);

        l_req->m_cmd.ref(new epp_Command(NULL,secdns_update,epp_trid("ABC-12345")));

        l_req->m_name.ref(new epp_string("acme.org"));

        epp_DomainUpdate_ref l_obj(new epp_DomainUpdate());

        l_obj->setRequestData(*l_req);

        test.processAction(l_obj);

        epp_DomainUpdateRsp_ref l_rsp = l_obj->getResponseData();

        printRsp(l_rsp);

        cout << "Remove command" << endl;

        SecDNSUpdate_ref secdns_update1(new SecDNSUpdate());

        secdns_update1->m_rem.ref(new SecDNSUpdateRem());
        secdns_update1->m_rem->m_key_tags.ref(new key_tag_seq());
        secdns_update1->m_rem->m_key_tags->push_back(kt1);
        
        l_req->m_cmd.ref(new epp_Command(NULL,secdns_update1,epp_trid("ABC-12345")));

        l_req->m_name.ref(new epp_string("acme.org"));

        l_obj->setRequestData(*l_req);

        test.processAction(l_obj);

        epp_DomainUpdateRsp_ref l_rsp1 = l_obj->getResponseData();

        printRsp(l_rsp1);        
 
        cout << "Change command" << endl;

        SecDNSUpdate_ref secdns_update2(new SecDNSUpdate());

        secdns_update2->m_chg.ref(new SecDNSUpdateChg());
        secdns_update2->m_chg->m_ds_data.ref(new SecDNSDsData_seq());
        secdns_update2->m_chg->m_ds_data->push_back(*ds_data2);
       
        l_req->m_cmd.ref(new epp_Command(NULL,secdns_update2,epp_trid("ABC-12345")));

        l_req->m_name.ref(new epp_string("acme.org"));
      
        l_obj->setRequestData(*l_req);
      
        test.processAction(l_obj);
      
        epp_DomainUpdateRsp_ref l_rsp2 = l_obj->getResponseData();
      
        printRsp(l_rsp2);
    }
    catch(epp_XMLException ex) 
    {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) 
    {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) 
    {
        cout << "Something else went wrong...\n";
    }

    
    try 
    {
        epp_DomainDeleteReq_ref l_req(new epp_DomainDeleteReq());

        l_req->m_cmd.ref(new epp_Command(NULL, NULL, epp_trid("ABC-12346"))); 
        l_req->m_name.ref(new epp_string("acme.org"));
      
        epp_DomainDelete_ref l_obj(new epp_DomainDelete());
      
        l_obj->setRequestData(*l_req);
      
        test.processAction(l_obj);
      
        epp_DomainDeleteRsp_ref l_rsp = l_obj->getResponseData();
    }
    catch(epp_XMLException ex) 
    {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) 
    {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) 
    {
        cout << "Something else went wrong...\n";
    }
   
}
