/************************************************************************
* EPP RTK C++ DomainProtocol Extension for .org
 * Copyright (C) 2001, 2002 The Global Name Registry, Limited 
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ************************************************************************/

/* $Header: /cvsroot/epp-rtk/liberty-rtk-addon/c++/org/src/main_transfertest.cc,v 1.2 2003/07/22 21:06:32 tubadanm Exp $ */

/**
 * This example shows the usage of the OxrsTransfer class in a Domain Transfer
 * request.  Most of this code is a copy of the main_domaintransfer.cc from
 * the directory above.  The code specific to this example is where OxrsTransfer
 * appears below.
 */
 
#include <string>
#include <iostream>

#include "epp-rtk-cpp/data/epp_eppdata.h"
#include "epp-rtk-cpp/data/epp_domainXMLbase.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainTransfer.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"
#include "OxrsTransfer.h"

using namespace std;
using namespace eppobject::epp;
using namespace eppobject::domain;


void printRsp(const epp_DomainTransferRsp_ref & req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart1(req->m_rsp);
      printResDataHeaders("domain", "trnData");
      
      if(req->m_trn_data != NULL) {
	 if(req->m_trn_data->m_name != NULL)
	   cerr << "<domain:name>" << *req->m_trn_data->m_name << "</domain:name>" << endl;
	 
	 if(req->m_trn_data->m_transfer_status != NULL) {
	    string trStatus = returnTransferStatusType(*req->m_trn_data->m_transfer_status);
	    cerr << "<domain:trStatus>" << trStatus << "</domain:trStatus>" << endl;
	 }
	 
	 if(req->m_trn_data->m_request_client_id != NULL)
	   cerr << "<domain:reID>" << *req->m_trn_data->m_request_client_id << "</domain:reID>" << endl;
	 
	 if(req->m_trn_data->m_request_date != NULL)
	   cerr << "<domain:reDate>" << *req->m_trn_data->m_request_date << "</domain:reDate>" << endl;
	 
	 if(req->m_trn_data->m_action_client_id != NULL)
	   cerr << "<domain:acID>" << *req->m_trn_data->m_action_client_id << "</domain:acID>" << endl;
	 
	 if(req->m_trn_data->m_action_date != NULL)
	   cerr << "<domain:acDate>" << *req->m_trn_data->m_action_date << "</domain:acDate>" << endl;
	 
	 if(req->m_trn_data->m_expiration_date != NULL)
	   cerr << "<domain:exDate>" << *req->m_trn_data->m_expiration_date << "</domain:exDate>" << endl;
      }
      
      printResDataFooter("domain", "trnData");
      printResponsePart2(req->m_rsp);
   }
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   string extra_usage = "[-extradata=trid=xxx:name=abc.tld:op=(query|request|cancel|approve|reject):authinfo=123123:period=1:reg=abc:contact1=t;abc:contact2=a;abc:contact3=b;abc]";
   if(!handleArgs(test, argc, argv, username, password, extra_usage))
     return 1;

   map<string,string> extraDataMap;
   
   if(argc > 1) {
      for(int parm = 1; parm < argc; parm++) {
	 
	 if (!strncmp(argv[parm], "-extradata=", 11)) {
            cout << "Found extra data param..." << endl;
	    string td = &argv[parm][11];
	    extraDataMap = getExtraDataMap(td);
	 }
      }
   }
   
   testLogIn(test, username, password);
   
   try {
      
      epp_DomainTransferReq_ref l_req(new epp_DomainTransferReq());
   
      string trID;
      if ( extraDataMap.find("trid") == extraDataMap.end() ) trID = "ABC-12345";
      else trID = extraDataMap["trid"];
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       NULL,
				       epp_trid(trID)));
      
      epp_DomainTransfer_ref l_obj(new epp_DomainTransfer());
      epp_DomainTransferRsp_ref l_rsp;
      
      l_req->m_trans.ref(new epp_TransferRequest());

      if ( extraDataMap.find("name") == extraDataMap.end() ) {
         l_req->m_name.ref(new epp_string("example.com"));
      } else if ( ! extraDataMap["name"].empty() ) {
         l_req->m_name.ref(new epp_string(extraDataMap["name"]));
      }

      if ( extraDataMap.find("op") == extraDataMap.end() ) {
	 l_req->m_trans->m_op.ref(new epp_TransferOpType(QUERY));
      } else if ( ! extraDataMap["op"].empty() ) {
	 l_req->m_trans->m_op.ref(new epp_TransferOpType(returnTransferType(extraDataMap["op"])));
      }

      if ( extraDataMap.find("authinfo") == extraDataMap.end() ) {
	 l_req->m_trans->m_auth_info.ref(new epp_AuthInfo());
	 l_req->m_trans->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
	 l_req->m_trans->m_auth_info->m_value.ref(new epp_string("2fooBAR"));
      } else if ( ! extraDataMap["authinfo"].empty() ) {
	 l_req->m_trans->m_auth_info.ref(new epp_AuthInfo());
	 l_req->m_trans->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
	 l_req->m_trans->m_auth_info->m_value.ref(new epp_string(extraDataMap["authinfo"]));
      }

      if ( extraDataMap.find("period") != extraDataMap.end() ) {
	 l_req->m_period.ref(new epp_DomainPeriod(YEAR, atoi(extraDataMap["period"].c_str())));
      }

      /*
       * Let's build the OxrsTransfer data so that
       * this RRP-based domain can be transferred to
       * my EPP-based registrar account...
       * .org domains in the RRP space have no bona fide
       * contacts -- when being transferred to an EPP registrar,
       * they need a registrant, an admin, a tech and a billing
       * contact (so, 4 in all).
       */
      OxrsTransfer_ref oxrsTransfer;
      oxrsTransfer.ref(new OxrsTransfer());
      if ( extraDataMap.find("authinfo") == extraDataMap.end() ) {
         oxrsTransfer->m_registrant.ref(new epp_string("jd1234"));
      } else {
         oxrsTransfer->m_registrant.ref(new epp_string(extraDataMap["reg"]));
      }
      oxrsTransfer->m_contacts.ref(new epp_domain_contact_seq());
      if ( extraDataMap.find("contact1") == extraDataMap.end() ) {
         /*
          * AT LEAST ONE OF EACH TYPE OF CONTACT MUST BE SPECIFIED
          */
         epp_DomainContact Cont1(ADMIN,  epp_string("sh8013"));
	 epp_DomainContact Cont2(TECH,   epp_string("sh8013"));
	 epp_DomainContact Cont3(BILLING,epp_string("sh8013"));
	 oxrsTransfer->m_contacts->push_back(Cont1);
	 oxrsTransfer->m_contacts->push_back(Cont2);
	 oxrsTransfer->m_contacts->push_back(Cont2);
      } else {
         char key[10]; // contactxx
	 for(unsigned int n = 1; n <= 10; n++) {
            sprintf(key,"contact%d",n);
            if ( extraDataMap.find(key) == extraDataMap.end() ) break;
	    epp_DomainContact contact;
            string type, value;
            if ( ! extraDataMap[key].empty() ) {
               // contact1=t/abc
               // contact2=a/qwerty
               // etc...
               type = extraDataMap[key].substr(0,1);
               value = extraDataMap[key].substr(2);
               if ( type == "t" ) contact.m_type.ref(new epp_DomainContactType(TECH));
               if ( type == "a" ) contact.m_type.ref(new epp_DomainContactType(ADMIN));
               if ( type == "b" ) contact.m_type.ref(new epp_DomainContactType(BILLING));
               contact.m_id.ref(new epp_string(value));
	       oxrsTransfer->m_contacts->push_back(contact);
            }
	 }
      }
      
      l_req->m_cmd->m_extentions.ref(new epp_extension_ref_seq());
      /*
       * Now push the extension onto the extension list so it's
       * in the request to the serer
       */
      l_req->m_cmd->m_extentions->push_back(oxrsTransfer);


      l_obj->setRequestData(*l_req);

      test.processAction(l_obj);

      l_rsp = l_obj->getResponseData();

#ifndef TEST_IO_REQUEST2CERR
      printRsp(l_rsp);
#endif

      cout << "\n----------------------------------\n";


      return 0;
      
   }
   catch(const epp_XMLException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(const epp_Exception & ex) {
      cout << "Exception: " << endl;
#ifndef TEST_IO_REQUEST2CERR
      cerr << "<response>" << endl;
      printResultsSeq(ex.m_details);
      printTransID(ex.m_trans_id);
      cerr << "</response>" << endl;
#endif
   }
   catch(const epptransport::epp_TrException & ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }
   
}
